
#include "kPGPDecrypt.h"
#include <stdlib.h>
#include "misc.h"
#include "myProfile.h"

const int maxRows = 12;

kPGPDecrypt :: kPGPDecrypt(const QString &infile, const QString &outfile,
			   char *optCmdLine, bool detachedSignature)
{
	setCaption(i18n("Decrypt / Verify"));

	widMain = new QWidget(this);
	setCentralWidget(widMain);

	layMain = new QVBoxLayout(widMain,MYSPACING);
	layR1 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR1);
	layR2 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR2);
	layR3 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR3);
        layR4 = new QHBoxLayout(MYSPACING);
        layMain->addLayout(layR4);
	layMain->addStretch(2);

	lblProgramToUse = new QLabel(i18n("Profile:"), widMain);
	lblProgramToUse->setFixedSize(lblProgramToUse->sizeHint());
	layR1->addWidget(lblProgramToUse);

	cbPGPVersion = new QComboBox(FALSE, widMain);
	cbPGPVersion->setMinimumSize(cbPGPVersion->sizeHint());
	cbPGPVersion->insertStrList(getProfileList());
	layR1->addWidget(cbPGPVersion);

	lblOpenFile = new QLabel(i18n("Incoming file: "), widMain);
	lblOpenFile->setFixedSize(lblOpenFile->sizeHint());
	layR2->addWidget(lblOpenFile);

	cmbFileIn = new QComboBox(TRUE,widMain);
	cmbFileIn->insertItem(i18n("<Clipboard>"));
	cmbFileIn->insertItem(i18n("<Other file...>"));
	cmbFileIn->setEditText("");
	cmbFileIn->setMinimumSize(cmbFileIn->sizeHint());
	connect(cmbFileIn,SIGNAL(activated(const QString &)),this,SLOT(cmbFileIn_activated(const QString &)));
	layR2->addWidget(cmbFileIn);

	chkDecrypt = new QCheckBox(i18n("Decrypt"),widMain);
	chkDetachedSig = new QCheckBox(i18n("Detached Signature"),widMain);
	chkDecrypt->setMinimumSize(chkDecrypt->sizeHint());
	chkDetachedSig->setMinimumSize(chkDetachedSig->sizeHint());
	connect(chkDecrypt,SIGNAL(clicked()),this,SLOT(chkDecrypt_clicked()));
	connect(chkDetachedSig,SIGNAL(clicked()),this,SLOT(chkDetachedSig_clicked()));
	layR3->addWidget(chkDecrypt);
	layR3->addWidget(chkDetachedSig);

	if (detachedSignature) chkDetachedSig->setChecked(true);

	layR4->addSpacing(MYSPACING*6);

	lblSaveAs = new QLabel(i18n("Save as: "), widMain);
	lblSaveAs->setFixedSize(lblSaveAs->sizeHint()); // CALL ME WHEN I CHANGE!
	layR4->addWidget(lblSaveAs);

	cmbFileOut = new QComboBox(TRUE,widMain);
	cmbFileOut->insertItem(i18n("<Clipboard>"));
	cmbFileOut->insertItem(i18n("<Other file...>"));
	cmbFileOut->setMinimumSize(cmbFileOut->sizeHint());
	cmbFileOut->setEditText("");
	connect(cmbFileOut,SIGNAL(activated(const QString &)),this,SLOT(cmbFileOut_activated(const QString &)));
	layR4->addWidget(cmbFileOut);

	widOkCancel = new QWidget(widMain);
	layMain->addWidget(widOkCancel);
	setupButtons(widOkCancel,TRUE,TRUE,FALSE);

	connect (getOkButton(), SIGNAL(clicked()), this, SLOT(cmdDoItclicked()));
	connect (getCancelButton(), SIGNAL(clicked()), this, SLOT(cmdCancel_clicked()));

	if (!infile.isEmpty()) {
		if (infile.compare("--clipboard") == 0)
			cmbFileIn->setEditText(i18n("<Clipboard>"));
		else
			cmbFileIn->setEditText(infile);

		if (!outfile.isEmpty()) {
			if (outfile.compare("--clipboard") == 0)
				cmbFileOut->setEditText(i18n("<Clipboard>"));
			else
				cmbFileOut->setEditText(outfile);
		} else {
			QString s(infile), t, u;
			int iStart;

			iStart = s.findRev('.');
			if (iStart > 0) {
				t = s.right(s.length() - iStart).lower();
				u = s.left(iStart);
			} else
				t = " "; // won't match my formulas below...good

			if (t.compare(".pgp") == 0 || t.compare(".asc") == 0 || t.compare(".gpg") == 0) cmbFileOut->setEditText(u);
			else cmbFileOut->setEditText(infile);
		}
	} else {
		if (!outfile.isEmpty()) {
			if (outfile.compare("--clipboard") == 0)
				cmbFileOut->setEditText(i18n("<Clipboard>"));
			else
				cmbFileOut->setEditText(outfile);
		}
	}

	layMain->activate();
	resize(370,200);
	setupMenubar();
	chkDecrypt_clicked();

	switch (cbPGPVersion->count()) {
		case 0: {
				// No profiles!
				mnuFileSelected(ID_KPGPSHELL_FILE_PREFS);
				close(); // need to use a profile!
				break;
			}
		case 1: {
				break;
			}
		default: {
				cbPGPVersion->insertItem(i18n("(none selected)"),0);
				setToDefaultProfile(cbPGPVersion);
				break;
			}
	}
	qApp->processEvents();
}

void kPGPDecrypt::cmbFileIn_activated(const QString &a) {
	// if (strcasecmp(i18n("<Other file...>"),a) == 0) {
        if (i18n("<Other file...>") == a) {
		QString f = KFileDialog::getOpenFileName(0, 0, this, 0);
		if (!f.isNull()) {
			cmbFileIn->insertItem(f.data(),0);
			cmbFileIn->setCurrentItem(0);
		}
	}
}

void kPGPDecrypt::cmbFileOut_activated(const QString &a) {
        // if (strcasecmp(i18n("<Other file...>"),a) == 0) {
        if (i18n("<Other file...>") == a) {
		QString f = KFileDialog::getSaveFileName(0, 0, this, 0);
		if (!f.isNull()) {
			cmbFileOut->insertItem(f.data(),0);
			cmbFileOut->setCurrentItem(0);
		}
	}
}

void kPGPDecrypt :: cmdDoItclicked() {
	myClipboard *tmpClip = NULL;
	myProfile *tmpProfile = NULL;
	bool usingIncomingClipboard = FALSE;
	bool usingOutgoingClipboard = FALSE;
	QString *tmp;
	int retVal = 0;

	if (cbPGPVersion->currentText() == i18n("(none selected)")) return;

	tmpProfile = new myProfile(cbPGPVersion->currentText());
	if (!tmpProfile) {
		showBadProfileDialog();
                return;
	}

	if (tmpProfile->pgpVersion == 0) {
		showBadProfileDialog();
		return;
	}

	// okay, we check for sane settings FIRST
	if (strlen(cmbFileIn->currentText()) < 1) {
		KMessageBox::error(this,("Sorry, but you must enter a incoming file to decrypt."));
		return;
	}

	if (chkDetachedSig->isChecked() || chkDecrypt->isChecked()) {
		if (strlen(cmbFileOut->currentText()) < 1) {
			KMessageBox::error(this, i18n("Sorry, but you must enter a outgoing file to decrypt the incoming file."));
			return;
		}
	}

	if (strcasecmp(cmbFileIn->currentText(),i18n("<Clipboard>")) == 0) usingIncomingClipboard = TRUE;
	if (strcasecmp(cmbFileOut->currentText(),i18n("<Clipboard>")) == 0) usingOutgoingClipboard = TRUE;

	if (usingIncomingClipboard || usingOutgoingClipboard) tmpClip = new myClipboard();

	// If we're using the clipboard, make an input file...
	if (usingIncomingClipboard == TRUE) {
		tmp = getSafeTmpNameQ();
		tmp->append(".1.asc");
		cmbFileIn->setEditText(tmp->data());
  		tmpClip->saveFromClipboard(tmp->data());
		delete tmp;
	}
	if (usingOutgoingClipboard == TRUE) {
		tmp = getSafeTmpNameQ();
		tmp->append(".2.asc");
		cmbFileOut->setEditText(tmp->data());
		delete tmp;
	}

	retVal = tmpProfile->decryptFile(cmbFileIn->currentText(),cmbFileOut->currentText(),(chkDetachedSig->isChecked()),(!chkDecrypt->isChecked()));

	if (usingIncomingClipboard == TRUE) unlink(cmbFileIn->currentText());

	if (usingOutgoingClipboard == TRUE) {
		tmpClip->saveIntoClipboard(cmbFileOut->currentText());
		unlink(cmbFileOut->currentText());
		if (chkDecrypt->isChecked()) {
			KMessageBox::information(this,i18n("If decryption was successful,\nplease paste back the result before hitting ok."));
		}
	}
	close();
}

void kPGPDecrypt::chkDecrypt_clicked()
{
	if (chkDecrypt->isChecked()) {
		chkDetachedSig->setChecked(false);
		cmbFileOut->show();
		lblSaveAs->setText(i18n("Save As:"));
		lblSaveAs->setFixedSize(lblSaveAs->sizeHint()); // CALL ME WHEN I CHANGE!
	} else {
		cmbFileOut->hide();
		lblSaveAs->setText("");
	}
}

void kPGPDecrypt::chkDetachedSig_clicked()
{
	QString tmp;
	if (chkDetachedSig->isChecked()) {
		chkDecrypt->setChecked(false);
		tmp = cmbFileOut->currentText();          // we have to change incoming file and signature
		cmbFileOut->setEditText(cmbFileIn->currentText());
		cmbFileIn->setEditText(tmp);
		cmbFileOut->show();
		lblSaveAs->setText(i18n("Signature:"));
		lblSaveAs->setFixedSize(lblSaveAs->sizeHint()); // CALL ME WHEN I CHANGE!
	} else {
		tmp = cmbFileOut->currentText();          // change back incoming file and signature
		cmbFileOut->setEditText(cmbFileIn->currentText());
		cmbFileIn->setEditText(tmp);
		cmbFileOut->hide();
		lblSaveAs->setText("");
	}
}

void kPGPDecrypt::cmdCancel_clicked()
{
	close();
}

