
#include "kPGPEncrypt.h"
#include "misc.h"
#include "kPGPPrefs.h"
#include "kPGPEncryptOptions.h"

kPGPEncrypt :: kPGPEncrypt(const QString &infile, const QString &outfile,
			   char *optCmdLine)
{
	if (optCmdLine) soptCmdLine.sprintf(" %s ",optCmdLine); else soptCmdLine = "";
	setCaption(i18n("Encrypt / Sign"));

	widMain = new QWidget(this);
	setCentralWidget(widMain);

	layMain = new QVBoxLayout(widMain,MYSPACING);
	layR1 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR1);
	layR2 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR2);
	layR3 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR3);
	layR4 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR4);
	layR5 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR5);
	layR6 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR6);
	layR7 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR7);
	layMain->addStretch(2);

	/* Note: labels and ... buttons are FIXED sizes, text boxes have
	 * only minimum sizes - cdw
	 */

	// ====================================================================
	layR1->addSpacing(MYSPACING);

	lblProgramToUse = new QLabel(i18n("Profile:"),widMain);
	lblProgramToUse->setFixedSize(lblProgramToUse->sizeHint());
	layR1->addWidget(lblProgramToUse);

	cbPGPVersion = new QComboBox(FALSE, widMain);
	cbPGPVersion->setMinimumSize(cbPGPVersion->sizeHint());
	layR1->addWidget(cbPGPVersion);
	connect (cbPGPVersion, SIGNAL(activated(int)), this, SLOT(cbPGPVersion_changed(int)));

	layR1->addSpacing(MYSPACING);
	// ======================================================================
	layR2->addSpacing(MYSPACING);

	lblOpenFile = new QLabel(i18n("File: "), widMain);
	lblOpenFile->setFixedSize(lblOpenFile->sizeHint());
	layR2->addWidget(lblOpenFile);

	cmbFileIn = new QComboBox(TRUE,widMain);
	cmbFileIn->insertItem(i18n("<Clipboard>"));
	cmbFileIn->insertItem(i18n("<Other file...>"));
	cmbFileIn->setEditText("");
	cmbFileIn->setMinimumSize(cmbFileIn->sizeHint());
	connect(cmbFileIn,SIGNAL(activated(const QString &)),this,SLOT(cmbFileIn_activated(const QString &)));
	layR2->addWidget(cmbFileIn);

	layR2->addSpacing(MYSPACING);
	// ======================================================================
	layR3->addSpacing(MYSPACING);

	lblSaveAs = new QLabel(i18n("Save as: "), widMain);
	lblSaveAs->setFixedSize(lblSaveAs->sizeHint());
	layR3->addWidget(lblSaveAs);

	cmbFileOut = new QComboBox(TRUE,widMain);
	cmbFileOut->insertItem(i18n("<Clipboard>"));
	cmbFileOut->insertItem(i18n("<Other file...>"));
	cmbFileOut->setEditText("");
	cmbFileOut->setMinimumSize(cmbFileOut->sizeHint());
	connect(cmbFileOut,SIGNAL(activated(const QString &)),this,SLOT(cmbFileOut_activated(const QString &)));
	layR3->addWidget(cmbFileOut);

	layR3->addSpacing(MYSPACING);
	// ======================================================================
	layR4->addSpacing(MYSPACING);

	chkSign = new QCheckBox(i18n("Sign as: "), widMain);
	chkSign->setFixedSize(chkSign->sizeHint());
	layR4->addWidget(chkSign);
	connect(chkSign,SIGNAL(clicked()),SLOT(chkSign_clicked()));
	chkSign->setChecked(TRUE);

	cmbSecretKey = new QComboBox(FALSE, widMain);
	cmbSecretKey->setMinimumSize(cmbSecretKey->sizeHint());
	layR4->addWidget(cmbSecretKey);

	layR4->addSpacing(MYSPACING);
	// ======================================================================
	layR5->addSpacing(MYSPACING);

	layR5->addSpacing(chkSign->sizeHint().width() + MYSPACING);

	chkDetachSign =  new QCheckBox(i18n("Detach Signature"), widMain);
	chkDetachSign->setMinimumSize(chkDetachSign->sizeHint());
	layR5->addWidget(chkDetachSign);
	connect(chkDetachSign,SIGNAL(clicked()),SLOT(chkDetachSign_clicked()));
	if (soptCmdLine.find(" -b ") != -1) chkDetachSign->setChecked(TRUE);
	if (soptCmdLine.find(" --detach-sign ") != -1) chkDetachSign->setChecked(TRUE);

	chkClearSign = new QCheckBox(i18n("Clearsign"),widMain);
	chkClearSign->setMinimumSize(chkClearSign->sizeHint());
	layR5->addWidget(chkClearSign);
	connect(chkClearSign,SIGNAL(clicked()),SLOT(chkClearSign_clicked()));
	chkClearSign->setChecked(FALSE);
	if (soptCmdLine.find(" --clearsign ") != -1) chkClearSign->setChecked(TRUE);

	chkUniversalText = new QCheckBox(i18n("Universal Text"),widMain);
	chkUniversalText->setMinimumSize(chkUniversalText->sizeHint());
	layR5->addWidget(chkUniversalText);
	connect(chkUniversalText,SIGNAL(clicked()),SLOT(chkUniversalText_clicked()));
	chkUniversalText->setChecked(TRUE);
	if (soptCmdLine.find(" -t ") != -1) chkUniversalText->setChecked(TRUE);
	if (soptCmdLine.find(" --textmode ") != -1) chkUniversalText->setChecked(TRUE);

	layR5->addSpacing(MYSPACING);
	// ======================================================================
	layR6->addSpacing(MYSPACING);

	chkEncrypt = new QCheckBox(i18n("Encrypt to:"),widMain);
	chkEncrypt->setFixedSize(chkEncrypt->sizeHint());
	layR6->addWidget(chkEncrypt,0,AlignTop);
	connect(chkEncrypt,SIGNAL(clicked()),SLOT(chkEncrypt_clicked()));

	list = new kPGPKeyList(widMain,KPGPKEYS_HIDETYPE + KPGPKEYS_HIDETRUST);
	list->setShowNamesOnly(TRUE);
	list->setAllowMultipleKeys(TRUE);
	list->setFlatView(TRUE);
	list->setShowOnlyGoodKeys(TRUE);
	list->setMinimumSize(cmbSecretKey->sizeHint().width(),cmbSecretKey->sizeHint().height()*6);
	layR6->addWidget(list);

	layR6->addSpacing(MYSPACING);
	// ======================================================================
	layR7->addSpacing(MYSPACING);

	layR7->addSpacing(chkEncrypt->sizeHint().width() + MYSPACING);

	chkEncrypt2Self = new QCheckBox(i18n("Encrypt To Self"),widMain);
	chkEncrypt2Self->setMinimumSize(chkEncrypt2Self->sizeHint());
	layR7->addWidget(chkEncrypt2Self);
	connect(chkEncrypt2Self,SIGNAL(clicked()),SLOT(chkEncrypt2Self_clicked()));
	chkEncrypt2Self->setChecked(false);

	chkASCIIArmor = new QCheckBox(i18n("ASCII Armor"),widMain);
	chkASCIIArmor->setMinimumSize(chkASCIIArmor->sizeHint());
	layR7->addWidget(chkASCIIArmor);
	connect(chkASCIIArmor,SIGNAL(clicked()),SLOT(chkASCIIArmor_clicked()));
	chkASCIIArmor->setChecked(TRUE);
	if (soptCmdLine.find(" -a ") != -1) chkASCIIArmor->setChecked(TRUE);
	if (soptCmdLine.find(" --armor ") != -1) chkASCIIArmor->setChecked(TRUE);

	layR7->addSpacing(MYSPACING);
	// ======================================================================

	widOkCancel = new QWidget(widMain);
	layMain->addWidget(widOkCancel);
//	setupButtons(widOkCancel,TRUE,TRUE,FALSE,TRUE);
	setupButtons(widOkCancel,TRUE,TRUE,FALSE,FALSE);

	connect (getOkButton(), SIGNAL(clicked()), this, SLOT(cmdDoItclicked()));
	connect (getCancelButton(), SIGNAL(clicked()), this, SLOT(cmdCancelclicked()));
//	connect (getOptionsButton(),SIGNAL(clicked()),SLOT(cmdOptions_clicked()));

//	options = new QStrList();
	currentProfile = NULL;
	qlistSecKeyIDs = new QStrList();
	qlistKeyIDs = new QStrList();

	if (!infile.isEmpty()) {
		if (infile.lower().compare("--clipboard") == 0)
			cmbFileIn->setEditText(i18n("<Clipboard>"));
		else
			cmbFileIn->setEditText(infile);

		if (!outfile.isEmpty()) {
			if (outfile.lower().compare("--clipboard") == 0)
				cmbFileOut->setEditText(i18n("<Clipboard>"));
			else
				cmbFileOut->setEditText(outfile);
		} else {
			cmbFileOut->setEditText(infile +".pgp");
		}
	}

	layMain->activate();
	resize(490, 400);
	setupMenubar();
	show();

	cbPGPVersion->insertStrList(getProfileList());
	switch (cbPGPVersion->count()) {
		case 0: {
				// No profiles!
				mnuFileSelected(ID_KPGPSHELL_FILE_PREFS);
				close(); // need to use a profile!
				break;
			}
		case 1: {
				cbPGPVersion_changed(0);
				break;
			}
		default: {
				cbPGPVersion->insertItem(i18n("(none selected)"),0);
				setToDefaultProfile(cbPGPVersion);
				if (cbPGPVersion->currentItem() > 0) cbPGPVersion_changed(cbPGPVersion->currentItem());
				break;
		}
	}
	qApp->processEvents();
}

void kPGPEncrypt::cmbFileIn_activated(const QString &a) {
//	if (strcasecmp(i18n("<Other file...>"),a) == 0) {
        if (i18n("<Other file...>") == a) {
		QString f = KFileDialog::getOpenFileName(0, 0, this, 0);
		if (!f.isNull()) {
			cmbFileIn->setEditText(f.data());
			if (strlen(cmbFileOut->currentText()) > 0) {
				f += ".pgp";
				cmbFileOut->setEditText(f.data());
			}
		}
	}
}

void kPGPEncrypt::cmbFileOut_activated(const QString &a) {
//	if (strcasecmp(i18n("<Other file...>"),a) == 0) {
        if (i18n("<Other file...>") == a) {
		QString f = KFileDialog::getSaveFileName(0, 0, this, 0);
		if (!f.isNull()) cmbFileOut->setEditText(f.data());
	}
}

void kPGPEncrypt :: cmdDoItclicked() {
	QString pgpAppString, tmpString;
	QString *tmp = new QString("");
	QString *recipList = new QString("");
	QStrList *recipList2;
	bool usingIncomingClipboard = FALSE;
	bool usingOutgoingClipboard = FALSE;
	int pgpVersion=0, retVal=0;
	unsigned int i=0;
	myClipboard *tmpClip = NULL;


	if (!currentProfile) {
		showBadProfileDialog();
                return;
	}

	if (strlen(cmbFileOut->currentText()) < 2) {
		KMessageBox::error(this,i18n("Sorry, but you must enter a outgoing file to save as."));
		return;
	}

	if (strlen(cmbFileIn->currentText()) < 2) {
		KMessageBox::error(this,i18n("Sorry, but you must enter a incoming file to encrypt."));
		return;
	}

	if (!chkSign->isChecked() && !chkEncrypt->isChecked()) {
		KMessageBox::error(this,i18n("Sorry, but you must sign and/or encrypt!"));
		return;
	}

	if (currentProfile->pgpVersion == 0) {
		KMessageBox::error(this,i18n("There was an error with the profile you selected. Please select another, or close and re-open this window."));
		return;
	}

	pgpVersion = currentProfile->pgpVersion;

	if (strcasecmp(i18n("<Clipboard>"),cmbFileIn->currentText()) == 0) usingIncomingClipboard = TRUE;
	if (strcasecmp(i18n("<Clipboard>"),cmbFileOut->currentText()) == 0) usingOutgoingClipboard = TRUE;

	if (usingIncomingClipboard || usingOutgoingClipboard) tmpClip = new myClipboard();

	// If we're using the clipboard, make an input file...
	if (usingIncomingClipboard == TRUE) {
		tmp = getSafeTmpNameQ();
		tmp->append(".1.asc");
		cmbFileIn->setEditText(tmp->data());
	  	if (tmpClip->saveFromClipboard(tmp->data()) == FALSE) {
			KMessageBox::sorry(this,i18n("Could not retrieve anything from clipboard."));
	  		return;
	  	}
		delete tmp;
	}
	if (usingOutgoingClipboard == TRUE) {
		tmp = getSafeTmpNameQ();
		tmp->append(".2.asc");
		cmbFileOut->setEditText(tmp->data());
		delete tmp;
	}

	if (chkEncrypt->isChecked()) {
		recipList2 = list->getSelectedList();
		for (i=0;i<recipList2->count();i++) {
			tmpString.sprintf(currentProfile->recipFormat.data(),recipList2->at(i));
			recipList->append(tmpString.data());
		}
		if (chkEncrypt2Self->isChecked()) {
			tmpString.sprintf(currentProfile->recipFormat.data(),qlistSecKeyIDs->at(cmbSecretKey->currentItem()));
			recipList->append(tmpString.data());
		}
		delete recipList2;
	}

	retVal = currentProfile->encryptFile(cmbFileIn->currentText(),cmbFileOut->currentText(),qlistSecKeyIDs->at(cmbSecretKey->currentItem()),recipList->data(),chkDetachSign->isChecked(),
		chkSign->isChecked(),chkEncrypt->isChecked(),chkASCIIArmor->isChecked(),chkUniversalText->isChecked(),chkClearSign->isChecked());

	delete recipList;
	if (usingIncomingClipboard == TRUE) unlink(cmbFileIn->currentText());

	if (usingOutgoingClipboard == TRUE) {
		tmpClip->saveIntoClipboard(cmbFileOut->currentText());
		unlink(cmbFileOut->currentText());
		KMessageBox::information(this,i18n("If decryption was successful,\nplease paste back the result before hitting ok."));
	}
	close();
}

void kPGPEncrypt::cmdCancelclicked()
{
	close();
}

void kPGPEncrypt::cbPGPVersion_changed(int listIndex) {
	pkiKey *thisKey = NULL;
	QList<pkiName> namering;
	unsigned int i=0;
	QString tmpString;

	if (cbPGPVersion->currentText() == i18n("(none selected)")) return;
	if (cbPGPVersion->currentItem() == 0 && cbPGPVersion->count() > 1) return;

	if (qlistSecKeyIDs->count() > 0) qlistSecKeyIDs->clear();

	if (currentProfile != NULL) delete currentProfile;

	currentProfile = new myProfile(cbPGPVersion->text(listIndex).data());
	currentProfile->loadKeyring();

	list->setProfile(currentProfile);
	list->loadKeyring();

	if (cmbSecretKey->count() > 0) cmbSecretKey->clear();

	for (i=0;i<currentProfile->keyring.count();i++) {
		// Per KEY
		thisKey = currentProfile->keyring.at(i);
		namering = thisKey->names;
		if (thisKey->haveSecret && !thisKey->isRevoked && !thisKey->isExpired) {
			qlistSecKeyIDs->append(thisKey->getKeyID()->data());
			if (thisKey->names.count() < 1)
				tmpString.sprintf("%s (%s)",i18n("????").data(),thisKey->getKeyType()->data());
			else
				tmpString.sprintf("%s <%s> (%s)",thisKey->names.at(0)->getDisplayName()->data(),thisKey->names.at(0)->getEmailAddress()->data(),thisKey->getKeyType()->data());
			cmbSecretKey->insertItem(tmpString.data());
		}
 	}
}

/* ***********************************************************************
 * Please note: when in a function that is for the checkbox it references
 * it is NOT necessary to re-call yourself if you uncheck yourself, unless
 * you actually deal with that condition. Otherwise, if you change the
 * checked status of ANY checkbox in this class, call the _clicked() func.
 *
 *                                                Chris Wiegand
 *
 * P.S. Before changing the checked status, and calling the requisite
 * function, check the status and save us some time and recursion!
 * ***********************************************************************
 */

void kPGPEncrypt::chkSign_clicked() {
	if (!chkSign->isChecked()) {
		// turn off sub-settings...
		if (chkClearSign->isChecked()) {
			chkClearSign->setChecked(FALSE);
			chkClearSign_clicked();
		}
		if (chkDetachSign->isChecked()) {
			chkDetachSign->setChecked(FALSE);
			chkDetachSign_clicked();
		}
		if (chkUniversalText->isChecked()) {
			chkUniversalText->setChecked(FALSE);
			chkUniversalText_clicked();
		}
	} else {
		// turned on - default settings
		if (!chkUniversalText->isChecked()) {
			chkUniversalText->setChecked(TRUE);
			chkUniversalText_clicked();
		}
	}
}

void kPGPEncrypt::chkDetachSign_clicked()
{
	if (chkDetachSign->isChecked()) {
		// turned ON
		if (chkEncrypt->isChecked()) {
			switch (KMessageBox::questionYesNo(this,i18n("Sorry"),i18n("You cannot detach a signature and encrypt at the same time. Which do you want to do?"),i18n("Detach Signature"),i18n("Encrypt"))) {
				case KMessageBox::Yes: // detach sign
					chkEncrypt->setChecked(FALSE);
					chkEncrypt_clicked();
					break;
				default: // encrypt or cancel
					chkDetachSign->setChecked(FALSE);
					return;
					break;
			}
		}

		if (chkClearSign->isChecked()) {
			switch (KMessageBox::questionYesNo(this,i18n("You cannot clearsign and detach the signature at the same time. Which one do you want to do?"),i18n("Detach Signature"),i18n("Clearsign"))) {
				case KMessageBox::Yes: // detach sign
					chkClearSign->setChecked(FALSE);
					chkClearSign_clicked();
					chkUniversalText->setChecked(FALSE);
					chkUniversalText_clicked();
					break;
				default: // clearsign or cancel
					chkDetachSign->setChecked(FALSE);
					return;
					break;
			}
		}

		// just to be sure
		if (!chkSign->isChecked()) {
			chkSign->setChecked(TRUE);
			chkSign_clicked();
		}
	}

	if (chkDetachSign->isChecked()) {
		// still turned ON
		chkUniversalText->setChecked(FALSE);
		chkUniversalText_clicked();
	}
}

void kPGPEncrypt::chkClearSign_clicked()
{
	if (chkClearSign->isChecked()) {
		// turned ON
		if (chkEncrypt->isChecked()) {
			switch (KMessageBox::questionYesNo(this,i18n("You cannot clearsign and encrypt at the same time. Which do you want to do?"),i18n("Clearsign"),i18n("Encrypt"))) {
				case KMessageBox::Yes:  // clearsign
					chkEncrypt->setChecked(FALSE);
					chkEncrypt_clicked();
					break;
				default:  // encrypt or "cancel"
					chkClearSign->setChecked(FALSE);
					return;
					break;
			}
		}

		if (chkDetachSign->isChecked()) {
			switch (KMessageBox::questionYesNo(this,i18n("You cannot clearsign and detach the signature at the same time. Which one do you want to do?"),i18n("Clearsign"),i18n("Detach Signature"))) {
				case KMessageBox::Yes:  // clearsign
					chkDetachSign->setChecked(FALSE);
					chkDetachSign_clicked();
					break;
				default:  // detach sign or "cancel"
					chkClearSign->setChecked(FALSE);
					return;
					break;
			}
		}

		// just to be sure
		if (!chkSign->isChecked()) {
			chkSign->setChecked(TRUE);
			chkSign_clicked();
		}
	}

	if (chkClearSign->isChecked()) {
		// still turned ON
		chkUniversalText->setChecked(TRUE);
		chkUniversalText_clicked();
		chkASCIIArmor->setChecked(TRUE);
		chkASCIIArmor_clicked();
	}
}

void kPGPEncrypt::chkUniversalText_clicked()
{
	if (!chkUniversalText->isChecked()) {
		// turned OFF
		if (chkClearSign->isChecked()) {
			switch (KMessageBox::questionYesNo(this,i18n("To clear sign a document, I must also have Universal Text on. Turning it off will turn off clear sign. Do you want to turn it off anyways?"))) {
				case KMessageBox::Yes: // yes
					chkClearSign->setChecked(FALSE);
					chkClearSign_clicked();
					break;
				default: // no or cancel
					chkUniversalText->setChecked(TRUE);
					chkUniversalText_clicked(); // I call myself because we want to make sure detach sign is turned off anyways...
					return;
					break;
			}
		}
	} else {
		if (chkDetachSign->isChecked()) {
			switch (KMessageBox::questionYesNo(this,i18n("You could not use Universal Text when making a Detached Signature .Turning it on will turn off Detached Signature. Do you want to turn it on anyways?"))) {
				case KMessageBox::Yes: // yes
					chkDetachSign->setChecked(FALSE);
					chkDetachSign_clicked();
					break;
				default: // no or cancel
					chkUniversalText->setChecked(FALSE);
					return;
					break;
			}
		}

		// just to be sure
		if (!chkSign->isChecked()) {
			chkSign->setChecked(TRUE);
			chkSign_clicked();
		}

	}
}

void kPGPEncrypt::chkEncrypt_clicked()
{
	if (chkEncrypt->isChecked()) {
		// turned ON
		if (chkDetachSign->isChecked()) {
			switch (KMessageBox::questionYesNo(this,i18n("You cannot detach a signature and encrypt at the same time. Which do you want to do?"),i18n("Encrypt"),i18n("Detach Signature"))) {
				case KMessageBox::Yes:  // encrypt
					chkDetachSign->setChecked(FALSE);
					chkDetachSign_clicked();
					break;
				default:  // detach sig or "cancel"
					chkEncrypt->setChecked(FALSE);
					chkEncrypt_clicked(); // recurse, please.
					return;
					break;
			}
		}

		if (chkClearSign->isChecked()) {
			switch (KMessageBox::questionYesNo(this,i18n("You cannot clearsign and encrypt at the same time. Which do you want to do?"),i18n("Encrypt"),i18n("Clearsign"))) {
				case KMessageBox::Yes:  // encrypt
					chkClearSign->setChecked(FALSE);
					chkClearSign_clicked();
					break;
				default:  // clearsign or "cancel"
					chkEncrypt->setChecked(FALSE);
					chkEncrypt_clicked(); // recurse, please
					return;
					break;
			}
		}

		if (!chkEncrypt2Self->isChecked()) {
			chkEncrypt2Self->setChecked(TRUE); // default setting
			chkEncrypt2Self_clicked();
		}
	} else {
		if (chkEncrypt2Self->isChecked()) {
			chkEncrypt2Self->setChecked(FALSE); // not avail
			chkEncrypt2Self_clicked();
		}
	}
}

void kPGPEncrypt::chkASCIIArmor_clicked()
{
	if (!chkASCIIArmor->isChecked()) {
		// turned OFF
		if (chkClearSign->isChecked()) {
			switch (KMessageBox::questionYesNo(this,i18n("To clear sign a document, I must also have ASCII Armor on. Turning it off will turn off clear sign. Do you want to turn it off anyways?"))) {
				case KMessageBox::Yes: // yes
					chkClearSign->setChecked(FALSE);
					chkClearSign_clicked();
					break;
				default: // no or cancel
					chkASCIIArmor->setChecked(TRUE);
					return;
					break;
			}
		}
	}
}

void kPGPEncrypt::chkEncrypt2Self_clicked()
{
	if (chkEncrypt2Self->isChecked()) {
		// turned on...
		if (!chkEncrypt->isChecked()) {
			chkEncrypt->setChecked(TRUE);
			chkEncrypt_clicked(); // this may cause Problems -- Stefan
		}
	}
}

void kPGPEncrypt::cmdOptions_clicked()
{
//	if (!currentProfile) {
//                KMsgBox::message(this,i18n("Error"),i18n("There was an error with the profile you selected. Please select another, or close and re-open this window."), KMsgBox::EXCLAMATION);
//                return;
//	}
//
//	kPGPEncryptOptions *a = new kPGPEncryptOptions(currentProfile,options);
//	a->show();
}
