
#include "kPGPKeyAdd.h"
#include "misc.h"
#include "myProfile.h"

kPGPKeyAdd :: kPGPKeyAdd(myProfile *passedProfile, kPGPKeyList *passedKPGPKeyList, const char *keyID1) {
	mainFun(passedProfile,passedKPGPKeyList);
	if (keyID1) setKeyID(keyID1);
}

void kPGPKeyAdd::setKeyID(const char *c)
{
	txtKSKeyID->setText(c);
	chkKeyserver->setChecked(TRUE);
}

void kPGPKeyAdd::setFile(const char *c)
{
	txtFile->setText(c);
	chkUseFile->setChecked(TRUE);
}

void kPGPKeyAdd::setClipboard()
{
	chkClipboard->setChecked(TRUE);
}

void kPGPKeyAdd::mainFun(myProfile *passedProfile, kPGPKeyList *passedKPGPKeyList) {
	setCaption(i18n("Add/Import Key..."));
	tmpProfile = passedProfile;
	winPGPKeyWindow = passedKPGPKeyList;
	grpMain = new QButtonGroup(this);
	grpMain->hide();
	grpMain->setExclusive(TRUE);

	widMain = new QWidget(this);
	setCentralWidget(widMain);

	layMain = new QVBoxLayout(widMain,5);

	// Using a file?
	layR1 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR1);

	chkUseFile = new QRadioButton(i18n("From a file"),widMain);
	chkUseFile->setFixedSize(chkUseFile->sizeHint());
	grpMain->insert(chkUseFile);
	layR1->addWidget(chkUseFile);
	txtFile = new QLineEdit(widMain);
	txtFile->setMinimumSize(txtFile->sizeHint());
	layR1->addWidget(txtFile);

	cmdChooseFile = new QPushButton("...", widMain);
	connect (cmdChooseFile, SIGNAL(clicked()), this, SLOT(cmdChooseFile_clicked()));
 	cmdChooseFile->setFixedSize(cmdChooseFile->sizeHint());
	layR1->addWidget(cmdChooseFile);

	// Using a Key Server? (only if available!)
	layR2 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR2);
	chkKeyserver = new QRadioButton(i18n("From a keyserver, import email or keyID"),widMain);
	chkKeyserver->setFixedSize(chkKeyserver->sizeHint());
	grpMain->insert(chkKeyserver);
	layR2->addWidget(chkKeyserver);

	txtKSKeyID = new QLineEdit(widMain);
	txtKSKeyID->setMinimumSize(txtKSKeyID->sizeHint());
	layR2->addWidget(txtKSKeyID);

	// Using the clipboard?
	layR3 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR3);
	chkClipboard = new QRadioButton(i18n("From the clipboard"),widMain);
	chkClipboard->setFixedSize(chkClipboard->sizeHint());
	grpMain->insert(chkClipboard);
	layR3->addWidget(chkClipboard);
	lblBlank = new QLabel(this);
	lblBlank->setMinimumSize(lblBlank->sizeHint());
	layR3->addWidget(lblBlank);

	layR4 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR4);

	widOkCancel = new QWidget(widMain);
	layR4->addWidget(widOkCancel);
	setupButtons(widOkCancel,TRUE,TRUE,FALSE);

	connect(getCancelButton(),SIGNAL(clicked()),this,SLOT(cmdCancel_clicked()));
	connect(getOkButton(), SIGNAL(clicked() ), this, SLOT(cmdDoIt_clicked() ) );

	layMain->activate();
	resize(400, 120);
}

void kPGPKeyAdd :: cmdChooseFile_clicked() {
	QString f = KFileDialog::getSaveFileName(0, 0, this, 0 );
	if (!f.isNull() ) txtFile->setText(f );
}

void kPGPKeyAdd :: cmdDoIt_clicked() {
  bool okWeDidIt;
  okWeDidIt = false;
	if (tmpProfile->pgpVersion == 0) {
		showBadProfileDialog();
		return;
	}

	// okay, we check for sane settings FIRST
	if (chkKeyserver->isChecked()) {
		// CAN we even use a keyserver?
		if (!tmpProfile->keyServerCompatible) {
			KMessageBox::sorry(this,i18n("Sorry, but your encryption program does not support using keyservers. Please choose a file or another program."));
			return;
		}
		// Okay, we can use it, but do we have a keyID?
		if (strlen(txtKSKeyID->text()) < 3) {
			KMessageBox::sorry(this,i18n("Please enter a Key ID or Email Address to search for."));
			return;
		}
		// Okay, let's do it!
		hide();
		if (!txtKSKeyID->isEnabled()) 
			okWeDidIt = (tmpProfile->importKey_Server(keysList) == 0);
		else
			okWeDidIt = (tmpProfile->importKey_Server(txtKSKeyID->text()) == 0);
	} else if (chkClipboard->isChecked()) {
		hide();
		okWeDidIt = (tmpProfile->importKey_Clipboard() == 0);
	} else {
		// Okay, check for a filename.
		if (strlen(txtFile->text()) < 3) {
			KMessageBox::sorry(this,i18n("Please choose a file to import."));
			return;
		}
		// Okay, let's do it!
		hide();
		okWeDidIt = (tmpProfile->importKey_File(txtFile->text()) == 0);
	}
	if (okWeDidIt) {
		if (winPGPKeyWindow) winPGPKeyWindow->loadKeyring();
		close();
	} else {
	  show();
	}
}

void kPGPKeyAdd::cmdCancel_clicked()
{
	close();
}

