
#include "kPGPKeyExport.h"
#include "misc.h"
#include "myProfile.h"

kPGPKeyExport :: kPGPKeyExport(myProfile *passedProfile, const char *keyID1) {
	setCaption(i18n("Export Key..."));
	tmpProfile = passedProfile;

	grpMain = new QButtonGroup(this);
	grpMain->hide();
	grpMain->setExclusive(TRUE);

	widMain = new QWidget(this);
	setCentralWidget(widMain);

	layMain = new QVBoxLayout(widMain,5);

	// Using a file?
	layR1 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR1);

	chkUseFile = new QRadioButton(i18n("To a file"),widMain);
	chkUseFile->setFixedSize(chkUseFile->sizeHint());
	grpMain->insert(chkUseFile);
	layR1->addWidget(chkUseFile);
	txtFile = new QLineEdit(widMain);
	txtFile->setMinimumSize(txtFile->sizeHint());
	layR1->addWidget(txtFile);

	cmdChooseFile = new QPushButton("...", widMain);
	connect (cmdChooseFile, SIGNAL(clicked()), this, SLOT(cmdChooseFile_clicked()));
 	cmdChooseFile->setFixedSize(cmdChooseFile->sizeHint());
	layR1->addWidget(cmdChooseFile);

	// Using a Key Server? (only if available!)
	layR2 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR2);
	chkKeyserver = new QRadioButton(i18n("To a keyserver"),widMain);
	chkKeyserver->setFixedSize(chkKeyserver->sizeHint());
	grpMain->insert(chkKeyserver);
	layR2->addWidget(chkKeyserver);

	txtKSKeyID = new QLineEdit(widMain);
	layR2->addWidget(txtKSKeyID);
	if (keyID1) txtKSKeyID->setText(keyID1);

	// Using the clipboard?
	layR3 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR3);
	chkClipboard = new QRadioButton(i18n("To the clipboard"),widMain);
	chkClipboard->setFixedSize(chkClipboard->sizeHint());
	grpMain->insert(chkClipboard);
	layR3->addWidget(chkClipboard);
	lblBlank = new QLabel(this);
	lblBlank->setMinimumSize(lblBlank->sizeHint());
	layR3->addWidget(lblBlank);

	layR4 = new QHBoxLayout(MYSPACING);
	layMain->addLayout(layR4);

	widOkCancel = new QWidget(widMain);
	layR4->addWidget(widOkCancel);
	setupButtons(widOkCancel,TRUE,TRUE,FALSE);

	connect(getCancelButton(),SIGNAL(clicked()),this,SLOT(cmdCancel_clicked()));
	connect(getOkButton(), SIGNAL(clicked() ), this, SLOT(cmdDoIt_clicked() ) );

	layMain->activate();
	resize(450, ((cmdChooseFile->sizeHint().height() * 4)+ (MYSPACING * 3)));
}

void kPGPKeyExport :: cmdChooseFile_clicked() {
	QString f = KFileDialog::getSaveFileName(0, 0, this, 0 );
	if (!f.isNull() ) txtFile->setText(f );
}

void kPGPKeyExport :: cmdDoIt_clicked() {
	if (tmpProfile->pgpVersion == 0) {
		showBadProfileDialog();
		return;
	}

	// okay, we check for sane settings FIRST
	if (chkKeyserver->isChecked()) {
		// CAN we even use a keyserver?
		if (!tmpProfile->keyServerCompatible) {
			KMessageBox::sorry(this,i18n("Sorry, but your encryption program does not support using keyservers. Please choose a file or another program."));
			return;
		}
		// Okay, we can use it, but do we have a keyID?
		if (strlen(txtKSKeyID->text()) < 3) {
			KMessageBox::sorry(this,i18n("Please enter a Key ID or Email Address to export."));
			return;
		}
		// Okay, let's do it!
		tmpProfile->exportKey_Server(txtKSKeyID->text());
	} else if (chkClipboard->isChecked()) {
		tmpProfile->exportKey_Clipboard(txtKSKeyID->text());
	} else {
		// Okay, check for a filename.
		if (strlen(txtFile->text()) < 3) {
			KMessageBox::sorry(this,i18n("Please choose a file to import."));
			return;
		}
		// Okay, let's do it!
		tmpProfile->exportKey_File(txtKSKeyID->text(),txtFile->text());
	}
}

void kPGPKeyExport::cmdCancel_clicked()
{
	close();
}

