
#include "kPGPKeyList.h"
#include "kPGPKeyAdd.h"
#include "kPGPKeyExport.h"
#include "kPGPPrefs.h"
#include "kPGPSelectKey.h"

kPGPKeyList :: kPGPKeyList(QWidget *parent, int colOptions) : QWidget(parent) {
	QString sFile;
	KConfig *cfg = NULL;
	KIconLoader ldr;

	currentProfile = NULL;
	flatView = FALSE;
	showOnlySecretKeys = FALSE;
	showNamesOnly = FALSE;
	showKeysOnly = FALSE;
	allowMultipleKeys = FALSE;
	showOnlyGoodKeys = FALSE;

	mnuKeys = new QPopupMenu();
	mnuPKey = new QPopupMenu();
	mnuSKey = new QPopupMenu();
	mnuSigs = new QPopupMenu();

	/* ************************************************************************************
	 * Load Menubar
	 */

	mnuKeys->insertItem(i18n("&Generate Key Pair..."), ID_GENKEY);

	mnuKeys->insertItem(i18n("&Import Key..."), ID_IMPORTKEY);
	mnuSKey->insertItem(i18n("&Update Key..."), ID_IMPORTKEY);
	mnuPKey->insertItem(i18n("&Update Key(s)..."), ID_IMPORTKEY);
	mnuSigs->insertItem(i18n("&Import Key..."), ID_IMPORTKEY);

	mnuKeys->insertItem(i18n("&Export Public Key..."), ID_EXPORTKEY);
	mnuSKey->insertItem(i18n("&Export Public Key..."), ID_EXPORTKEY);
	mnuPKey->insertItem(i18n("&Export Public Key..."), ID_EXPORTKEY);

	mnuKeys->insertItem(i18n("Key I&nfo..."), ID_KEYINFO);
	mnuSKey->insertItem(i18n("Key I&nfo..."), ID_KEYINFO);
	mnuPKey->insertItem(i18n("Key I&nfo..."), ID_KEYINFO);
	mnuSigs->insertItem(i18n("Signing Key I&nfo..."), ID_KEYINFO);

	mnuKeys->insertItem(i18n("&Edit Key..."), ID_EDITKEY);
	mnuSKey->insertItem(i18n("&Edit Key..."), ID_EDITKEY);
	mnuPKey->insertItem(i18n("&Edit Key..."), ID_EDITKEY);
	mnuSigs->insertItem(i18n("&Edit Signing Key..."), ID_EDITKEY);

	mnuKeys->insertItem(i18n("&Disable/Enable Key..."), ID_DISENABLEKEY);
	mnuSKey->insertItem(i18n("&Disable/Enable Key..."), ID_DISENABLEKEY);
	mnuPKey->insertItem(i18n("&Disable/Enable Key..."), ID_DISENABLEKEY);

	mnuKeys->insertItem(i18n("Re&move Key..."), ID_REMOVEKEY);
	mnuSKey->insertItem(i18n("Re&move Key..."), ID_REMOVEKEY);
	mnuPKey->insertItem(i18n("Re&move Key..."), ID_REMOVEKEY);

	mnuKeys->insertItem(i18n("Re&voke Key..."), ID_REVOKEKEY);
	mnuSKey->insertItem(i18n("Re&voke Key..."), ID_REVOKEKEY);

	mnuKeys->insertSeparator();
	mnuSKey->insertSeparator();
	mnuPKey->insertSeparator();

	mnuKeys->insertItem(i18n("&Add Name..."), ID_ADDNAME);
	mnuSKey->insertItem(i18n("&Add Name..."), ID_ADDNAME);

	mnuKeys->insertItem(i18n("&Sign Key..."), ID_SIGNKEY);
	mnuSKey->insertItem(i18n("&Sign Key..."), ID_SIGNKEY);
	mnuPKey->insertItem(i18n("&Sign Key..."), ID_SIGNKEY);

	mnuKeys->insertItem(i18n("Revo&ke Signature..."), ID_REVOKESIG);
	mnuSigs->insertItem(i18n("Revo&ke Signature..."), ID_REVOKESIG);

	mnuKeys->insertSeparator();

	mnuKeys->insertItem(i18n("&Reload Keyring"), ID_RELOADKEYRING);

	connect(mnuKeys,SIGNAL(activated(int)),SLOT(mnuKeys_selected(int)));
	connect(mnuSKey,SIGNAL(activated(int)),SLOT(mnuKeys_selected(int)));
	connect(mnuPKey,SIGNAL(activated(int)),SLOT(mnuKeys_selected(int)));
	connect(mnuSigs,SIGNAL(activated(int)),SLOT(mnuKeys_selected(int)));
	
	qApp->processEvents();

	cfg = kapp->config();
	if (!cfg) return;

	/* ***************************************************************8
	 * Load pixmaps
	 */

	pixKeySec = ldr.loadIcon("icon_key_sec.xpm",KIcon::User);
	pixKeyPub = ldr.loadIcon("icon_key_pub.xpm",KIcon::User);
	pixKeyPub2 = ldr.loadIcon("icon_key_pub2.xpm",KIcon::User);
	pixKeyExp = ldr.loadIcon("icon_key_expired.xpm",KIcon::User);
	pixKeyRev = ldr.loadIcon("icon_key_revoked.xpm",KIcon::User);
	pixKeyDis = ldr.loadIcon("icon_key_disabled.xpm",KIcon::User);

	pixNameSec = ldr.loadIcon("icon_name_sec.xpm",KIcon::User);
	pixNamePub = ldr.loadIcon("icon_name_pub.xpm",KIcon::User);
	pixNamePub2 = ldr.loadIcon("icon_name_pub2.xpm",KIcon::User);

	pixSigSec = ldr.loadIcon("icon_sig_sec.xpm",KIcon::User);
	pixSigPub = ldr.loadIcon("icon_sig_pub.xpm",KIcon::User);
	pixSigPub2 = ldr.loadIcon("icon_sig_pub2.xpm",KIcon::User);

	pixTrust0 = ldr.loadIcon("trust_0.xpm",KIcon::User);
	pixTrust1 = ldr.loadIcon("trust_1.xpm",KIcon::User);
	pixTrust2 = ldr.loadIcon("trust_2.xpm",KIcon::User);
	pixTrust3 = ldr.loadIcon("trust_3.xpm",KIcon::User);
	pixTrust4 = ldr.loadIcon("trust_4.xpm",KIcon::User);

	pixValid0 = ldr.loadIcon("valid_0.xpm",KIcon::User);
	pixValid1 = ldr.loadIcon("valid_1.xpm",KIcon::User);
	pixValid2 = ldr.loadIcon("valid_2.xpm",KIcon::User);
	pixValid3 = ldr.loadIcon("valid_3.xpm",KIcon::User);
	pixValid4 = ldr.loadIcon("valid_4.xpm",KIcon::User);

	/* ************************************************************************
	 * Load Profiles / Prefs
	 */
	
	qApp->processEvents();
	cfg->setGroup("__Key Management");
	showName = cfg->readBoolEntry("Name",TRUE);
	showEmail = cfg->readBoolEntry("Email",TRUE);
	showSize = cfg->readBoolEntry("Size",TRUE);
	showKeyID = cfg->readBoolEntry("KeyID",TRUE);
	showType = cfg->readBoolEntry("Type",TRUE);
	showTrust = cfg->readBoolEntry("Trust",TRUE);
	showValidity = cfg->readBoolEntry("Validity",TRUE);
	
	// Now, override from constructor...
	showName = (!(colOptions & KPGPKEYS_HIDENAME));
	showEmail = (!(colOptions & KPGPKEYS_HIDEEMAIL));
	showSize = (!(colOptions & KPGPKEYS_HIDESIZE));
	showKeyID = (!(colOptions & KPGPKEYS_HIDEKEYID));
	showType = (!(colOptions & KPGPKEYS_HIDETYPE));
	showTrust = (!(colOptions & KPGPKEYS_HIDETRUST));
	showValidity = (!(colOptions & KPGPKEYS_HIDEVALIDITY));

	qApp->processEvents();

	layTT = new QVBoxLayout(this);

	lstKeyring = new QListView(this);
	if (showName) lstKeyring->addColumn(i18n("Name"));
	if (showEmail) lstKeyring->addColumn(i18n("Email"));
	if (showSize) lstKeyring->addColumn(i18n("Size"));
	if (showKeyID) lstKeyring->addColumn(i18n("KeyID"));
	if (showType) lstKeyring->addColumn(i18n("Type"));
	if (showTrust)  lstKeyring->addColumn(i18n("Trust"));
	if (showValidity)  lstKeyring->addColumn(i18n("Validity"));
	lstKeyring->setMultiSelection(allowMultipleKeys);
	lstKeyring->setAllColumnsShowFocus(TRUE);
	connect(lstKeyring,SIGNAL(rightButtonClicked(QListViewItem *,const QPoint &,int)),
		this,SLOT(lstKeyring_rightClick(QListViewItem *,const QPoint &,int)));
	layTT->addWidget(lstKeyring);
	layTT->activate();

	setAcceptDrops(true);
	cfg->setGroup("__Global");
	flatView = (!cfg->readBoolEntry("KeyTreeView",TRUE));

	qApp->processEvents();
}

QPopupMenu *kPGPKeyList::getMenu() {
	return mnuKeys;
}

void kPGPKeyList::setFlatView(bool value) {
	flatView = value;
}

void kPGPKeyList::setShowOnlyGoodKeys(bool value) {
	showOnlyGoodKeys = value;
}

void kPGPKeyList::setShowOnlySecretKeys(bool value) {
	showOnlySecretKeys = value;
}

void kPGPKeyList::setShowNamesOnly(bool value) {
	showNamesOnly = value;
}

void kPGPKeyList::setShowKeysOnly(bool value) {
	showKeysOnly = value;
}

void kPGPKeyList::setAllowMultipleKeys(bool value) {
	allowMultipleKeys = value;
}

QSize kPGPKeyList::sizeHint() {
	// Reimplemented from QWidget to re-direct to lstKeyring
	return lstKeyring->sizeHint();
}

void kPGPKeyList::loadKeyring()
{
	// This profile requires that a good profile is loaded!
	QListViewItem *lviKey = NULL, *lviName = NULL, *lviSig = NULL;
	QList<pkiName> namering;
	QList<pkiSubKey> subkeys;
	QList<pkiSig> sigring;

	pkiKey *thisKey = NULL;       // Do NOT "delete" these objects, it would break the keyring!
	pkiName *thisName = NULL;     //  |
	pkiSubKey *thisSubKey = NULL; //  |
	pkiSig *thisSig = NULL;       //  v
	pkiKey *signKey = NULL;       // thru here...

	unsigned int i=0, j=0, k=0;
	QString *keyIconName = new QString();
	QString *strUnknown = new QString(i18n("Unknown"));
	QString *str4Q = new QString(i18n("????"));
	QProgressDialog *winProgress = NULL;
	QListViewItem *lviValid, *lviDunno, *lviSecretKeys, *lviRevokedExpiredDisabled;
	QListViewItem *lviOne;

	qApp->processEvents();

	if (currentProfile == NULL) return;
	if (currentProfile->pgpVersion == 0) return;

	qApp->processEvents();

	lstKeyring->clear();
	lstKeyring->setRootIsDecorated(!flatView);
	lstKeyring->setMultiSelection(allowMultipleKeys);
	lstKeyring->setEnabled(FALSE);

	if (flatView) {
		lviSecretKeys = lviRevokedExpiredDisabled = lviValid = lviDunno = NULL;
	} else {
		lviValid = new QListViewItem(lstKeyring,i18n("Known Valid Keys"));
		lviValid->setPixmap(0,pixKeyPub2);
		lviDunno = new QListViewItem(lstKeyring,i18n("Possibly Valid Keys"));
		lviDunno->setPixmap(0,pixKeyPub);
		lviSecretKeys = new QListViewItem(lstKeyring,i18n("Secret Key Pairs"));
		lviSecretKeys->setPixmap(0,pixKeySec);
		lviRevokedExpiredDisabled = new QListViewItem(lstKeyring,i18n("Revoked / Expired / Disabled Keys"));
		lviRevokedExpiredDisabled->setPixmap(0,pixKeyRev);
	}

	winProgress = new QProgressDialog(i18n("Loading keyring..."),0,currentProfile->keyring.count());

	for (i=0;i<currentProfile->keyring.count();i++) {
		// Per KEY
		winProgress->setProgress(i);
		qApp->processEvents();

		thisKey = currentProfile->keyring.at(i);
		if (thisKey->haveSecret)
			keyIconName->sprintf(i18n("%s Key Pair"),thisKey->getKeyType()->data());
		else
			keyIconName->sprintf(i18n("%s Public Key"),thisKey->getKeyType()->data());
		namering = thisKey->names;
		subkeys = thisKey->subkeys;

		lviOne = lviDunno; // default...
		if (thisKey->getValidity() >= 2) lviOne = lviValid;
		if (thisKey->haveSecret) lviOne = lviSecretKeys;
		if (thisKey->isRevoked || thisKey->isExpired || thisKey->isDisabled) lviOne = lviRevokedExpiredDisabled;
		qApp->processEvents();

		// Okay, here, we start to process keys...
		// We disqualify until it passes all tests...
		if ((thisKey->isRevoked || thisKey->isExpired || thisKey->isDisabled) && showOnlyGoodKeys) {
			// do nothing...
		} else if (showOnlySecretKeys && !thisKey->haveSecret) {
			// do nothing
		} else {
			if (!showNamesOnly) {
				// if we're not just showing names (used in encryption dialog), then show keys, names, and sigs...
				if (namering.count() > 0)
					lviKey = addLine(lviOne,namering.at(0)->getDisplayName(),namering.at(0)->getEmailAddress(),thisKey->getKeySize(),thisKey->getKeyID(),keyIconName->data(),thisKey->getTrust(),thisKey->getValidity());
				else
					lviKey = addLine(lviOne,strUnknown,str4Q,thisKey->getKeySize(),thisKey->getKeyID(),keyIconName->data(),thisKey->getTrust(),thisKey->getValidity());
		
				qApp->processEvents();
				if (thisKey->isRevoked)
					lviKey->setPixmap(0,pixKeyRev);
				else if (thisKey->isExpired)
					lviKey->setPixmap(0,pixKeyExp);
				else if (thisKey->isDisabled)
					lviKey->setPixmap(0,pixKeyDis);
				else if (thisKey->haveSecret)
					lviKey->setPixmap(0,pixKeySec);
				else if (thisKey->getValidity() >= 2)
					lviKey->setPixmap(0,pixKeyPub2);
				else
					lviKey->setPixmap(0,pixKeyPub);
				
				qApp->processEvents();
			}
	
			if (!showKeysOnly) {
				for (j=0;j<namering.count();j++) {
					// Per NAME
					qApp->processEvents();
		
					thisName = namering.at(j);
					sigring = thisName->sigs;
		
					if (showNamesOnly) 
						lviName = addLine(NULL,thisName->getDisplayName(),thisName->getEmailAddress(),thisKey->getKeySize(),thisKey->getKeyID(),i18n("Display Name"),-1,thisKey->getValidity());
					else
						lviName = addLine(lviKey,thisName->getDisplayName(),thisName->getEmailAddress(),thisKey->getKeySize(),thisKey->getKeyID(),i18n("Display Name"),-1,thisKey->getValidity());

					qApp->processEvents();
					if (thisKey->haveSecret) {
						lviName->setPixmap(0,pixNameSec);
					} else {
						if (thisKey->getValidity() > 2) lviName->setPixmap(0,pixNamePub2);
						else lviName->setPixmap(0,pixNamePub);
					}
		
					if (!showNamesOnly) {
						for (k=0;k<sigring.count();k++) {
							// Per SIGNATURE
							qApp->processEvents();
			
							thisSig = sigring.at(k);
							signKey = currentProfile->getKey(thisSig->getSigningKeyID()->data());
							if (signKey != NULL) {
								if (signKey->haveSecret) {
									keyIconName->sprintf(i18n("Personal Signature"));
								} else {
									if (signKey->getValidity() > 2 && signKey->getTrust() > 2)
									keyIconName->sprintf(i18n("Trusted Signature"));
								else
									keyIconName->sprintf(i18n("Untrusted Signature"));
							}
		
							if (signKey->names.count() > 0)
								lviSig = addLine(lviName,signKey->names.at(0)->getDisplayName(),signKey->names.at(0)->getEmailAddress(),signKey->getKeySize(),signKey->getKeyID(),keyIconName->data(),signKey->getTrust(),signKey->getValidity());
							else
								lviSig = addLine(lviName,strUnknown,str4Q,signKey->getKeySize(),signKey->getKeyID(),keyIconName->data(),signKey->getTrust(),signKey->getValidity());
		
							if (signKey->haveSecret) {
								lviSig->setPixmap(0,pixSigSec);
							} else {
								if (signKey->getValidity() > 2 && signKey->getTrust() > 2)
									lviSig->setPixmap(0,pixSigPub2);
								else
									lviSig->setPixmap(0,pixSigPub);
								}
							} else {
								lviSig = addLine(lviName,strUnknown,str4Q,str4Q,thisSig->getSigningKeyID(),i18n("Unknown Signature"),0,0);
								lviSig->setPixmap(0,pixSigPub);
							}	
						}
					}					
				}
	
				if (!showNamesOnly) {
					for (j=0;j<subkeys.count();j++) {
						// Per SUBKEY
						qApp->processEvents();
			
						thisSubKey = subkeys.at(j);
						keyIconName->sprintf(i18n("%s Subkey"),thisSubKey->getKeyType()->data());
						sigring = thisSubKey->sigs;
						if (namering.count() > 0)
							lviName = addLine(lviKey,namering.at(0)->getDisplayName(),namering.at(0)->getEmailAddress(),thisSubKey->getKeySize(),thisSubKey->getKeyID(),keyIconName->data(),-1,thisKey->getValidity());
						else
							lviName = addLine(lviKey,strUnknown,str4Q,thisSubKey->getKeySize(),thisSubKey->getKeyID(),keyIconName->data(),-1,thisKey->getValidity());
		
						if (thisKey->haveSecret) {
							lviName->setPixmap(0,pixKeySec);
						} else {
							if (thisKey->getValidity() > 2) lviName->setPixmap(0,pixKeyPub2);
							else lviName->setPixmap(0,pixKeyPub);
						}
			
						for (k=0;k<sigring.count();k++) {
							// Per SIGNATURE
							qApp->processEvents();
			
							thisSig = sigring.at(k);
							signKey = currentProfile->getKey(thisSig->getSigningKeyID()->data());
							if (signKey != NULL) {
								if (signKey->haveSecret) {
									keyIconName->sprintf(i18n("Personal Signature"));
								} else {
									if (signKey->getValidity() > 2 && signKey->getTrust() > 2)
										keyIconName->sprintf(i18n("Trusted Signature"));
									else
										keyIconName->sprintf(i18n("Untrusted Signature"));
								}
			
								if (signKey->names.count() > 0)
									lviSig = addLine(lviName,signKey->names.at(0)->getDisplayName(),signKey->names.at(0)->getEmailAddress(),signKey->getKeySize(),signKey->getKeyID(),keyIconName->data(),signKey->getTrust(),signKey->getValidity());
								else
									lviSig = addLine(lviName,strUnknown,str4Q,signKey->getKeySize(),signKey->getKeyID(),keyIconName->data(),signKey->getTrust(),signKey->getValidity());
			
								if (signKey->haveSecret) {
									lviSig->setPixmap(0,pixSigSec);
								} else {
									if (signKey->getValidity() > 2 && signKey->getTrust() > 2)
										lviSig->setPixmap(0,pixSigPub2);
									else
										lviSig->setPixmap(0,pixSigPub);
								}
							} else {
								lviSig = addLine(lviName,i18n("Unknown Signature"),"",i18n("????"), thisSig->getSigningKeyID()->data(),i18n("Unknown Signature"),0,0);
								lviSig->setPixmap(0,pixSigPub);
							}
						}					
					}
				}
			}
		}
		qApp->processEvents();
		// END PER KEY LOOP
	}
	// Now, if any of the four top-level children are empty, remove them for a nice-looking window
	if (!flatView) {
		if (lviSecretKeys->childCount() == 0) delete lviSecretKeys;
		if (lviRevokedExpiredDisabled->childCount() == 0) delete lviRevokedExpiredDisabled;
		if (lviValid->childCount() == 0) delete lviValid;
		if (lviDunno->childCount() == 0) delete lviDunno;
	}
	delete strUnknown;
	delete str4Q;
	lstKeyring->setEnabled(TRUE);
	lstKeyring->show(); // just in case...
	delete winProgress;
}

QListViewItem *kPGPKeyList::addLine(QListViewItem *lvParent, QString *displayName, QString *emailAddress, QString *keySize, QString *keyID, const char *typeText, const int trust, const int validity) {
	return addLine(lvParent, displayName->data(), emailAddress->data(), keySize->data(), keyID->data(), typeText, trust, validity); 
}

QListViewItem *kPGPKeyList::addLine(QListViewItem *lvParent, const char *displayName, const char *emailAddress, const char *keySize, const char *keyID, const char *typeText, const int trust, const int validity)
{
	int z = 0;
	QListViewItem *lviKey = NULL;
	if (lvParent) // subkey, name, or signature
		lviKey = new QListViewItem(lvParent);
	else
		lviKey = new QListViewItem(lstKeyring);

	if (showName) { lviKey->setText(z,displayName); z++; }
	if (showEmail) { lviKey->setText(z,emailAddress); z++; }
	if (showSize) { lviKey->setText(z,keySize); z++; }
	if (showKeyID) { lviKey->setText(z,keyID); z++; }
	if (showType) { lviKey->setText(z,typeText); z++; }
	if (showTrust) {
		switch(trust) {
			case -1: // Trust not applicable
			        lviKey->setText(z,"");           // Dont try to wrap this string into i18n  -- stefan
				break;
			case 1:
				lviKey->setPixmap(z,pixTrust1);
				break;
			case 2:
				lviKey->setPixmap(z,pixTrust2);
				break;
			case 3:
				lviKey->setPixmap(z,pixTrust3);
				break;
			case 4:
				lviKey->setPixmap(z,pixTrust4);
				break;
			case 0:
			default:
				lviKey->setPixmap(z,pixTrust0);
				break;
		}
		z++;
	}
	if (showValidity) {
		switch(validity) {
			case 1:
				lviKey->setPixmap(z,pixValid1);
				break;
			case 2:
				lviKey->setPixmap(z,pixValid2);
				break;
			case 3:
				lviKey->setPixmap(z,pixValid3);
				break;
			case 4:
				lviKey->setPixmap(z,pixValid4);
				break;
			case 0:
			default:
				lviKey->setPixmap(z,pixValid0);
				break;
		}
		z++;
	}
	return lviKey;
}

void kPGPKeyList::importKey(const char *keyID)
{
	kPGPKeyAdd *tmpP;
	if (!currentProfile) {
		showBadProfileDialog();
	    return;
	}
	if (!keyID || strlen(keyID) < 4) // not a valid keyID!
		tmpP = new kPGPKeyAdd(currentProfile,this);
	else
		tmpP = new kPGPKeyAdd(currentProfile,this,keyID);
	tmpP->show();
}

void kPGPKeyList::editKey(const char *keyID)
{
	if (!currentProfile) {
		showBadProfileDialog();
		return;
	}
	if (!keyID) return; // not a valid keyID!
	if (strlen(keyID) < 4) return; // not a valid keyID!
	currentProfile->editKey(keyID);
	loadKeyring();
}

void kPGPKeyList::disEnableKey(const char *keyID)
{
	if (!currentProfile) {
		showBadProfileDialog();
	    return;
	}
	if (!keyID) return; // not a valid keyID!
	if (strlen(keyID) < 4) return; // not a valid keyID!
	currentProfile->disEnableKey(keyID);
	loadKeyring();
}

void kPGPKeyList::revokeSignature(const char *keyID)
{
	if (!currentProfile) {
		showBadProfileDialog();
	    return;
	}
	if (!keyID) return; // not a valid keyID!
	if (strlen(keyID) < 4) return; // not a valid keyID!
	currentProfile->revokeSignature(keyID);
	loadKeyring();
}

void kPGPKeyList::addName(const char *keyID)
{
	if (!currentProfile) {
		showBadProfileDialog();
	    return;
	}
	if (!keyID) return; // not a valid keyID!
	if (strlen(keyID) < 4) return; // not a valid keyID!
	currentProfile->addName(keyID);
	loadKeyring();
}

void kPGPKeyList::keyInfo(const char *keyID)
{
	pkiKey *tmpKey = NULL;
	kPGPKeyProps *newWin = NULL;

	if (!currentProfile) {
		showBadProfileDialog();
	    return;
	}
	if (!keyID) return; // not a valid keyID!
	if (strlen(keyID) < 4) return; // not a valid keyID!

	tmpKey = currentProfile->getKey(keyID);
	if (tmpKey) {
		newWin = new kPGPKeyProps(tmpKey);
		newWin->show();
	}
	return;
}

void kPGPKeyList::genKeyPair()
{
	if (!currentProfile) {
		showBadProfileDialog();
	    return;
	}
	currentProfile->genKeyPair();
	loadKeyring();
}

char *kPGPKeyList::getSelectedKeyID()
{
	// This function returns the KeyID of the selected item...
	// we can only have one selection - do it easy!
	QListViewItem *tmpItem = NULL;
	QString a("");
	char *ret = NULL;
	tmpItem = lstKeyring->currentItem();
	if (tmpItem != NULL) {
		a = tmpItem->text(getKeyIDColumn());
		if (a.length() < 3) a = "";
	}
	ret = (char *) myMalloc(a.length()+1);
	strncpy(ret,a.data(),a.length()+1);
	return ret; // FIXME: Return a QString! -- cdw
}

QStrList *kPGPKeyList::getSelectedList()
{
	// This function returns a string list of keyIDs selected
	// Only have to use it with multi-select mode, and if you can support
	// that many selections. Of course, you can use:
	// getSelectedList().item(0)
	int i = 0;
	QListViewItem *tmpItem = NULL;
	QStrList *ret = new QStrList();

	if (lstKeyring->isMultiSelection()) {
		// do this the hard way...
		tmpItem = lstKeyring->firstChild();
		if (tmpItem) {
			for (i=0;i<lstKeyring->childCount();i++) {
				if (tmpItem->isSelected()) ret->append(tmpItem->text(getKeyIDColumn()));
				if (tmpItem->childCount() > 0) traverseChildren(tmpItem,ret); // get stuff beneath us...
				tmpItem = tmpItem->nextSibling();
			}
		}
		// traverse the entire tree finding what's selected and what's not...
	} else {
		// we can only have one selection - do it easy!
		tmpItem = lstKeyring->currentItem();
		if (tmpItem != NULL) ret->append(tmpItem->text(getKeyIDColumn()));
	}
	return ret;
}

void kPGPKeyList::traverseChildren(QListViewItem *lvi, QStrList *add2Me) {
	int i = 0;
	QListViewItem *child = NULL;
	
	// traverse the entire tree finding what's selected and what's not...
	child = lvi->firstChild();
	if (child) {
		for (i=0;i<lvi->childCount();i++) {
			if (child->isSelected()) add2Me->append(child->text(getKeyIDColumn()));
			if (child->childCount() > 0) traverseChildren(child,add2Me); // get stuff beneath us...
			child = child->nextSibling();
		}
	}
}

void kPGPKeyList::exportKey(const char *keyID)
{
	if (!currentProfile) {
		showBadProfileDialog();
	    return;
	}
	if (!keyID) return; // not a valid keyID!
	if (strlen(keyID) < 4) return; // not a valid keyID!
	kPGPKeyExport *tmpP = new kPGPKeyExport(currentProfile,keyID);
	tmpP->show();
	// No update needed - we just exported!
}

void kPGPKeyList::removeKey(const char *keyID)
{
	if (!currentProfile) {
		showBadProfileDialog();
	    return;
	}
	if (!keyID) return; // not a valid keyID!
	if (strlen(keyID) < 4) return; // not a valid keyID!
	currentProfile->removeKey(keyID);
	loadKeyring();
}

void kPGPKeyList::signKey(const char *keyIDToSign)
{
	char *a = NULL;
	if (!currentProfile) {
		showBadProfileDialog();
	    return;
	}
	if (!keyIDToSign) return; // not a valid keyID!
	if (strlen(keyIDToSign) < 4) return; // not a valid keyID!
	a = getSelectKey();
	if (!a) return; // not a valid keyID!
	if (strlen(a) < 4) return; // not a valid keyID!
	currentProfile->signKey(keyIDToSign,a);
	loadKeyring();
}

char *kPGPKeyList::getSelectKey()
{
	char *retVal;
	QString *returnKeyID;
	kPGPSelectKey *tmpSelectKey = NULL;

	if (!currentProfile) {
		showBadProfileDialog();
	    return NULL;
	}
	tmpSelectKey = new kPGPSelectKey(currentProfile,true);
	tmpSelectKey->show();
	while (tmpSelectKey->isVisible()) qApp->processEvents();
	returnKeyID = tmpSelectKey->selectedKey();

	if (returnKeyID)
		if (!returnKeyID->isNull()) {
			retVal = (char *) myMalloc(returnKeyID->length()+10);
			strcpy(retVal,returnKeyID->data());
			return retVal;
		} else {
			return NULL;
		}
	else
		return NULL;
}

void kPGPKeyList::removeRevokeKey(const char *keyID=NULL)
{
	if (!currentProfile) {
		showBadProfileDialog();
 		return;
	}
	if (!keyID) return; // not a valid keyID!
	if (strlen(keyID) < 4) return; // not a valid keyID!
	currentProfile->removeRevokeKey(keyID);
	loadKeyring();
}

void kPGPKeyList::mnuKeys_selected(int menuID)
{
	/* Note: for now, the menu will only act on the one key you
	 * select. Eventually, it may support multiple keys.
	 * This was done because we have some issues with Qt's TreeView
	 * widget, and the fact that selected items can be hidden
	 * by closing the parent.... -- cdw 
	 */
	switch (menuID) {
		case ID_CLOSE:
			close();
			break;
		case ID_IMPORTKEY:
			importKey(getSelectedKeyID());
			break;
		case ID_GENKEY:
			genKeyPair();
			break;
		case ID_RELOADKEYRING:
			if (!currentProfile) break;
			lstKeyring->setEnabled(FALSE);
			currentProfile->loadKeyring(); // to load into profile's memory
			lstKeyring->setEnabled(TRUE);
			loadKeyring(); // to show on screen...
			break;
		case ID_EDITKEY:
			editKey(getSelectedKeyID());
			break;
		case ID_REMOVEKEY:
			removeKey(getSelectedKeyID());
			break;
		case ID_REVOKEKEY:
			removeRevokeKey(getSelectedKeyID());
			break;
		case ID_ADDNAME:
			addName(getSelectedKeyID());
			break;
		case ID_SIGNKEY:
			signKey(getSelectedKeyID());
			break;
		case ID_EXPORTKEY:
			exportKey(getSelectedKeyID());
			break;
		case ID_REVOKESIG:
			revokeSignature(getSelectedKeyID());
			break;
		case ID_KEYINFO:
			keyInfo(getSelectedKeyID());
			break;
		case ID_DISENABLEKEY:
			disEnableKey(getSelectedKeyID());
			break;
		default:
			KMessageBox::error(this,i18n("Sorry, but that function is not currently available. Please go to http://geheimnis.sourceforge.net and submit a bug report."));
			break;
	}
}

void kPGPKeyList::lstKeyring_rightClick(QListViewItem *lvi, const QPoint &pos, int column) {
	// first, change the selection so that we selected the thing we just
	// clicked on...
	lstKeyring->setSelected(lvi,TRUE);

	// now, are we on a secret key pair, or a public key, or what?
	if (lvi) {
		QString tmp;
		tmp = lvi->text(4);
		if (tmp.contains(i18n("Subkey")) > 0) lvi = lvi->parent();
		if (tmp.contains(i18n("Display Name")) > 0) lvi = lvi->parent();

		tmp = lvi->text(4);
		if (tmp.contains(i18n("Key Pair")) > 0) {
			mnuSKey->exec(pos);
		} else if (tmp.contains(i18n("Public Key")) > 0) {
			mnuPKey->exec(pos);
		} else if (tmp.contains(i18n("Signature")) > 0) {
			mnuSigs->exec(pos);
		} else mnuKeys->exec(pos);
	} else {
		mnuKeys->exec(pos);
	}
}

int kPGPKeyList::getKeyIDColumn()
{
	int z = 0;
	if (!showKeyID) return -1; // FAIL!!!! EVIL BAD NOT GOOD!!

	if (showName) z++;
	if (showEmail) z++;
	if (showSize) z++;
// We don't need to show the rest, I just coded it for completeness .. cdw
//	if (showKeyID) z++;
//	if (showType) z++;
//	if (showTrust) z++;
//	if (showValidity) z++;
// This is designed to mirror the addItem thing above -
// if you change the order, change it here too...	
	return z;
}

void kPGPKeyList::setProfile(myProfile *newP)
{
	currentProfile = newP;
}

void kPGPKeyList::dropEvent(QDropEvent *e) {
	kPGPKeyAdd *p = NULL;
	QString a("");

	// I **HATE** this code. --cdw 
	QStrList strings;
	if ( QUrlDrag::decode( e, strings ) ) {
	        QStringList files;
 		if (QUrlDrag::decodeLocalFiles( e, files ) ) {
		 	for (QStringList::Iterator i=files.begin(); i!=files.end(); ++i) {
				p = new kPGPKeyAdd(currentProfile);
				a = (*i);
				// ch = (*i).data();
				p->setFile(a.data());
			        p->show();
 	 	 	}
 	 	}
 	}
}


