// -*- c++ -*-

#ifndef KPGPKEYLIST_H
#define KPGPKEYLIST_H

#include "includes.h"
#include "defines.h"
#include "kPGPShell.h"

#include "myProfile.h"
#include "kPGPKeyProps.h"

#define ID_CLOSE 500
#define ID_GENKEY 501
#define ID_EDITKEY 502
#define ID_REMOVEKEY 503
#define ID_REVOKEKEY 504
#define ID_ADDNAME 505
#define ID_SIGNKEY 506
#define ID_REVOKESIG 507
#define ID_RELOADKEYRING 508
#define ID_IMPORTKEY 509
#define ID_EXPORTKEY 510
#define ID_KEYINFO 511
#define ID_DISENABLEKEY 512
#define ID_GETKEYSIGS 514

const int KPGPKEYS_HIDENONE = 0;
const int KPGPKEYS_HIDENAME = 1;
const int KPGPKEYS_HIDEEMAIL = 2;
const int KPGPKEYS_HIDESIZE = 4;
const int KPGPKEYS_HIDEKEYID = 8;
const int KPGPKEYS_HIDETYPE = 16;
const int KPGPKEYS_HIDETRUST = 32;
const int KPGPKEYS_HIDEVALIDITY = 64;


class kPGPKeyList : public QWidget {
		Q_OBJECT
		
	public:

		kPGPKeyList(QWidget *parent, int colOptions=0);
		QSize sizeHint();
		void loadKeyring();
		QPopupMenu *getMenu();
		void setProfile(myProfile *newProfile);

		void setFlatView(bool value=FALSE);
		void setShowOnlySecretKeys(bool value=FALSE);
		void setShowNamesOnly(bool value=FALSE);
		void setShowKeysOnly(bool value=FALSE);
		void setAllowMultipleKeys(bool value=FALSE);
		void setShowOnlyGoodKeys(bool value=FALSE);

		char *getSelectedKeyID();
		QStrList *getSelectedList();
		
	public slots:
		void mnuKeys_selected(int);
		void lstKeyring_rightClick(QListViewItem *lvi, const QPoint &pos, int column);
		void dropEvent(QDropEvent *e);

	private:
		// Child widgets
		QVBoxLayout *layTT;
		QPopupMenu *mnuSKey;
		QPopupMenu *mnuSigs;
		QPopupMenu *mnuPKey;
		QPopupMenu *mnuKeys;
		QFrame *lblTop;
		QListView *lstKeyring;
		myProfile *currentProfile;

		bool flatView;						// By default, all of these are OFF / FALSE.
		bool showOnlySecretKeys;
		bool showNamesOnly;
		bool showKeysOnly;
		bool allowMultipleKeys;		// Call loadKeyring() if you change them! -- cdw
		bool showName;						// By default, all of these are ON / TRUE
		bool showEmail;
		bool showSize;
		bool showKeyID;
		bool showType;
		bool showTrust;
		bool showValidity;
		bool showOnlyGoodKeys;
		
		// Private functions...
		void importKey(const char *keyID);
		void keyInfo(const char *keyID);
		void exportKey(const char *keyID);
		void editKey(const char *keyID);
		void removeKey(const char *keyID);
		void signKey(const char *keyID);
		void revokeSignature(const char *sigID);
		void disEnableKey(const char *keyID);
		void addName(const char *keyID);
		void genKeyPair();
		void revokeKey(const char *keyID);
		void removeRevokeKey(const char *keyID);

		void exportKey2Server(const char *keyID);
		void exportKeyFile(const char *keyID, const char *fileName);
		char *getSelectKey();
		int getKeyIDColumn();
		QListViewItem *addLine(QListViewItem *lvParent=NULL, const char *displayName=NULL, const char *emailAddress=NULL, const char *keySize=NULL, const char *keyID=NULL, const char *typeText=NULL, const int trust=0, const int validity=0);
		QListViewItem *addLine(QListViewItem *lvParent, QString *displayName, QString *emailAddress, QString *keySize, QString *keyID, const char *typeText, const int trust, const int validity);
		void traverseChildren(QListViewItem *lvi, QStrList *add2Me);

		// FIXME KDNDDropZone *myDNDZone;

		// Private pixmaps
		QPixmap pixKeySec;   // trusted key pair
		QPixmap pixKeyPub;   // untrusted key
		QPixmap pixKeyPub2;  // trusted key
		QPixmap pixKeyExp;   // expired key
		QPixmap pixKeyRev;   // revoked key
		QPixmap pixKeyDis;   // disabled key
		QPixmap pixNameSec;  // name on key pair
		QPixmap pixNamePub;  // untrusted name
		QPixmap pixNamePub2; // trusted name
		QPixmap pixSigSec;   // signature from trusted key pair
		QPixmap pixSigPub;   // untrusted signature
		QPixmap pixSigPub2;  // trusted signature
		
		QPixmap pixTrust0;
		QPixmap pixTrust1;
		QPixmap pixTrust2;
		QPixmap pixTrust3;
		QPixmap pixTrust4;
		
		QPixmap pixValid0;
		QPixmap pixValid1;
		QPixmap pixValid2;
		QPixmap pixValid3;
		QPixmap pixValid4;
};

#endif  // KPGPKEYLIST_H

