
#include "kPGPProfileEdit_GPG.h"

kPGPProfileEdit_GPG :: kPGPProfileEdit_GPG(const char *profileName2) {
     KConfig *cfg = NULL;
	QStrList tmpList;
	QString tmpString;
	unsigned int i = 0;
	QListViewItem *tmpItem;
	QWidget *widMain;

	if (!profileName2) {
		KMessageBox::sorry(this,i18n("Somehow, kPGPProfileEdit_GPG got called with NO profile name!\nPlease visit http://geheimnis.sourceforge.net and submit a bug report."));
		return;
	}
	
	widMain = new QWidget(this);
	setCentralWidget(widMain);
	profileName = profileName2; // profileName is a QString in the header file...
	setCaption(profileName);
	resize(475, 265);
	
	layMain = new QGridLayout(widMain,8,5,10,10);

	lblPGPVersion = new QLabel(i18n("This profile uses GnuPG 1.0x."),widMain);
	lblPGPVersion->setMinimumSize(lblPGPVersion->sizeHint());
	layMain->addMultiCellWidget(lblPGPVersion,0,0,0,4,AlignLeft);

	lblExecs = new QLabel(i18n("gpg:"),widMain);
	lblExecs->setMinimumSize(lblExecs->sizeHint());
	layMain->addWidget(lblExecs,1,0,AlignLeft);

	txtPGPBin = new QLineEdit(widMain);
	txtPGPBin->setMinimumSize(txtPGPBin->sizeHint());
	layMain->addMultiCellWidget(txtPGPBin,1,1,1,3,AlignRight);
	
	cmdChooseExec = new QPushButton(i18n("Edit"),widMain);
	connect(cmdChooseExec,SIGNAL(clicked()),this,SLOT(cmdChooseExec_clicked()));
	cmdChooseExec->setMinimumSize(cmdChooseExec->sizeHint());
	layMain->addWidget(cmdChooseExec,1,4,AlignLeft);
	
	lblKeyrings = new QLabel(i18n("Keyrings"),widMain);
	lblKeyrings->setMinimumSize(lblKeyrings->sizeHint());
	layMain->addWidget(lblKeyrings,2,0,AlignLeft);
	
	// Now, show the listbox for the keyrings...
	lstKeyrings = new QListView(widMain);
	lstKeyrings->addColumn(i18n("Path"));
	lstKeyrings->addColumn(i18n("Type"));
	lstKeyrings->setAllColumnsShowFocus(TRUE);
	layMain->addMultiCellWidget(lstKeyrings,2,4,1,4);

	cmdNewKR = new QPushButton(i18n("Add..."), widMain);
	connect (cmdNewKR, SIGNAL(clicked()), this, SLOT(cmdNewKR_clicked()));
	cmdNewKR->setMinimumSize(cmdNewKR->sizeHint());
	layMain->addWidget(cmdNewKR,3,0);
	
	cmdDelKR = new QPushButton(i18n("Remove"), widMain);
	connect (cmdDelKR, SIGNAL(clicked()), this, SLOT(cmdDelKR_clicked()));
	cmdDelKR->setMinimumSize(cmdDelKR->sizeHint());
	layMain->addWidget(cmdDelKR,4,0);

	lblKS = new QLabel(i18n("Keyserver:"),widMain);
	lblKS->setFixedSize(lblKS->sizeHint());
	layMain->addWidget(lblKS,5,0);

	txtKS = new QLineEdit(widMain);
	txtKS->setMinimumSize(txtKS->sizeHint());
	layMain->addMultiCellWidget(txtKS,5,5,1,4);

	chkHonorHTTPProxy = new QCheckBox(i18n("Honor HTTP Proxy"),widMain);
	layMain->addMultiCellWidget(chkHonorHTTPProxy,6,6,1,4);
	
	widOkCancel = new QWidget(widMain);
	layMain->addMultiCellWidget(widOkCancel,7,7,0,4);
	setupButtons(widOkCancel,TRUE,TRUE,FALSE);

	connect (getCancelButton(), SIGNAL(clicked()), this, SLOT(cmdClose_clicked()));
	connect (getOkButton(), SIGNAL(clicked()), this, SLOT(cmdSave_clicked()));

	layMain->activate();

	// Finally, load the data...
     cfg = kapp->config();
     if (cfg == NULL) return; // failed...

     cfg->setGroup(profileName);
	txtPGPBin->setText(cfg->readEntry("binGPG","gpg"));
	txtKS->setText(cfg->readEntry("Keyserver","search.keyserver.net"));
	
	if (cfg->readEntry("HonorHTTPProxy","0").compare("1") == 0) {
		chkHonorHTTPProxy->setChecked(true);
	} else {
		chkHonorHTTPProxy->setChecked(false);
	}
	
	cfg->readListEntry("PublicKeyring",tmpList,';');
	if (tmpList.count() == 0) tmpList.append("~/.gnupg/pubring.gpg");
	for (i=0;i<tmpList.count();i++) tmpItem = new QListViewItem(lstKeyrings,tmpList.at(i),i18n("Public"));

	tmpList.clear();
		
	cfg->readListEntry("SecretKeyring",tmpList,';');
	if (tmpList.count() == 0) tmpList.append("~/.gnupg/secring.gpg");
	for (i=0;i<tmpList.count();i++) tmpItem = new QListViewItem(lstKeyrings,tmpList.at(i),i18n("Secret"));

	setupMenubar();
}

void kPGPProfileEdit_GPG::cmdClose_clicked()
{
	close();
}

void kPGPProfileEdit_GPG::cmdChooseExec_clicked()
{
	QString t = KFileDialog::getOpenFileName(0,0,this,0);
	if (!t.isNull()) txtPGPBin->setText(t.data());
}

void kPGPProfileEdit_GPG::cmdNewKR_clicked()
{
	QListViewItem *tmpItem = NULL;
	QString f = KFileDialog::getOpenFileName( 0, 0, this, 0 );
	if (!f.isNull()) {
		if (KMessageBox::questionYesNo(this,i18n("Is this keyring public? (If you're not sure, answer Yes.)")) == KMessageBox::Yes)
			tmpItem = new QListViewItem(lstKeyrings,f.data(),i18n("Public"));
		else
			tmpItem = new QListViewItem(lstKeyrings,f.data(),i18n("Secret"));
	}
}

void kPGPProfileEdit_GPG::cmdDelKR_clicked()
{
	if (lstKeyrings->currentItem()) delete lstKeyrings->currentItem();
}

void kPGPProfileEdit_GPG::cmdSave_clicked()
{
     KConfig *cfg = NULL;
	QListViewItem *tmpItem;
     QString lstPKeys, lstSKeys;

     cfg = kapp->config();
     if (cfg == NULL) return; // failed...

     cfg->setGroup(profileName.data());
	cfg->writeEntry("binGPG",txtPGPBin->text());
	
	lstPKeys = "";
	lstSKeys = "";
	tmpItem = lstKeyrings->firstChild();
	while (tmpItem) {
		if (strcmp(tmpItem->text(1),i18n("Public")) == 0) {
			lstPKeys.append(tmpItem->text(0));
			lstPKeys.append(";");
		} else {
			lstSKeys.append(tmpItem->text(0));
			lstSKeys.append(";");
		}
		tmpItem = tmpItem->nextSibling();
	}
	if (chkHonorHTTPProxy->isChecked()) {
		cfg->writeEntry("HonorHTTPProxy","1");
	} else {
		cfg->writeEntry("HonorHTTPProxy","0");
	}
	cfg->writeEntry("PublicKeyring",lstPKeys.data());
	cfg->writeEntry("SecretKeyring",lstSKeys.data());
	cfg->writeEntry("Keyserver",txtKS->text());
	cfg->sync();
		
	close();
}







