
#include "kPGPShell.h"

#include "misc.h"

#include "kPGPDNDZone.h"
#include "kPGPDecrypt.h"
#include "kPGPEncrypt.h"
#include "kPGPKeys.h"
#include "kPGPPrefs.h"

kPGPShell::kPGPShell() {
	// set nice stuff...
	setCaption(MY_APP_NAME);
	resize(300, 120);
	return;
}

void kPGPShell::showAboutApplication() {
	QString s;
	s = MY_APP_NAME;
	s.append(i18n("\nWritten by Chris Wiegand and Stefan Suchi.\nCovered by GPL v2 or newer.\nhttp://geheimnis.sourceforge.net\nGeheimnis comes with ABSOLUTELY NO WARRANTY WHATSOEVER to the extent permitted by applicable law."));
	KMessageBox::information(this,s);
}

void kPGPShell::setupMenubar()
{
	mbrMain = menuBar();

	// ------------------------------------------

	mnuFile = new QPopupMenu;
	mnuFile->insertItem(i18n("&Encrypt/Sign..."), ID_KPGPSHELL_FILE_ENCRYPTSIGN);
	mnuFile->insertItem(i18n("&Decrypt/Verify..."), ID_KPGPSHELL_FILE_DECRYPTVERIFY);
	mnuFile->insertItem(i18n("&Key Management"), ID_KPGPSHELL_KEYS);
	mnuFile->insertItem(i18n("&Preferences"), ID_KPGPSHELL_FILE_GLOBALPREFS);
	mnuFile->insertSeparator();
	mnuFile->insertItem(i18n("&Close Window"),ID_KPGPSHELL_FILE_CLOSE);
	// mnuFile->insertSeparator();
	// mnuFile->insertItem(i18n("E&xit"), ID_KPGPSHELL_FILE_EXIT);
  
	// Connect things together
	connect (mnuFile, SIGNAL (activated (int)), this, SLOT (mnuFileSelected (int)));
	mbrMain->insertItem(i18n("&File"), mnuFile,0);

	// -------------------------------------------

	mbrMain->insertItem(i18n("&Help"), customHelpMenu(FALSE));

	// -------------------------------------------

	mbrMain->show();
}

void kPGPShell::enableCloseAll()
{
	mnuFile->insertSeparator();
	mnuFile->insertItem(i18n("Close All"), ID_KPGPSHELL_FILE_EXIT);
}

void kPGPShell::mnuFileSelected(int idMenu) {
	KMainWindow *tmpForm = NULL;

	switch (idMenu) {
	  case ID_KPGPSHELL_FILE_EXIT:
			exit(0);
			break;
	  case ID_KPGPSHELL_FILE_DECRYPTVERIFY:
			tmpForm = new kPGPDecrypt();
			if (tmpForm) tmpForm->show();
			break;
	  case ID_KPGPSHELL_FILE_DECRYPTVERIFYCB:
			tmpForm = new kPGPDecrypt("--clipboard","--clipboard");
			if (tmpForm) tmpForm->show();
			break;
	  case ID_KPGPSHELL_FILE_ENCRYPTSIGN:
			tmpForm = new kPGPEncrypt();
			if (tmpForm) tmpForm->show();
			break;
	  case ID_KPGPSHELL_FILE_ENCRYPTSIGNCB:
			tmpForm = new kPGPEncrypt("--clipboard","--clipboard");
			if (tmpForm) tmpForm->show();
			break;
	  case ID_KPGPSHELL_KEYS:
			tmpForm = new kPGPKeys();
			if (tmpForm) tmpForm->show();
			break;
	  case ID_KPGPSHELL_CHANGELOG:
			kapp->invokeHTMLHelp("changelog.html","");
			break;
	  case ID_KPGPSHELL_FILE_CLOSE:
			close();
			break;
	  case ID_KPGPSHELL_PROFILE_EDIT:
	  case ID_KPGPSHELL_FILE_PREFS:
			{
				kPGPPrefs *tt = new kPGPPrefs();
				if (tt) tt->show();
			}
		  	break;
	  default:
			KMessageBox::error(this, i18n("Sorry, but that function is not currently available. Please go to http://geheimnis.sourceforge.net and submit a bug report."));
			break;
	};
}

void kPGPShell::setupButtons(QWidget *parent, bool ok, bool cancel, bool close, bool options) {
	int x = 0,y = 0;
	QHBoxLayout *hboxBottom = new QHBoxLayout(parent);

	if (options) {
		cmdOptions = new QPushButton(i18n("Options..."),parent);
		if (cmdOptions->sizeHint().width() > x) x = cmdOptions->sizeHint().width();
		if (cmdOptions->sizeHint().height() > y) y = cmdOptions->sizeHint().height();
		hboxBottom->addWidget(cmdOptions);
	}
	hboxBottom->addStretch(1);
	hboxBottom->addSpacing(MYSPACING);	
	if (close) {	
		cmdClose = new QPushButton(i18n("Close"),parent);
		if (cmdClose->sizeHint().width() > x) x = cmdClose->sizeHint().width();
		if (cmdClose->sizeHint().height() > y) y = cmdClose->sizeHint().height();
		hboxBottom->addWidget(cmdClose);
	}
	if (cancel) {
		if (close) hboxBottom->addSpacing(MYSPACING);
		cmdCancel = new QPushButton(i18n("Close"),parent);
		if (cmdCancel->sizeHint().width() > x) x = cmdCancel->sizeHint().width();
		if (cmdCancel->sizeHint().height() > y) y = cmdCancel->sizeHint().height();
		hboxBottom->addWidget(cmdCancel);
	}
	if (ok) {
		if (close || cancel) hboxBottom->addSpacing(MYSPACING);
		cmdOk = new QPushButton(i18n("Ok"),parent);
		if (cmdOk->sizeHint().width() > x) x = cmdOk->sizeHint().width();
		if (cmdOk->sizeHint().height() > y) y = cmdOk->sizeHint().height();
		hboxBottom->addWidget(cmdOk);
	}

	if (options) cmdOptions->setFixedSize(x,y);
	if (close) cmdClose->setFixedSize(x,y);
	if (cancel) cmdCancel->setFixedSize(x,y);
	if (ok) cmdOk->setFixedSize(x,y);

	hboxBottom->addSpacing(MYSPACING);
	hboxBottom->activate();
	parent->setFixedHeight(y + MYSPACING);
	parent->show();
}

QPushButton *kPGPShell::getOkButton() { return cmdOk; }
QPushButton *kPGPShell::getCloseButton() { return cmdClose; }
QPushButton *kPGPShell::getCancelButton() { return cmdCancel; }
QPushButton *kPGPShell::getOptionsButton() { return cmdOptions; }

