#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <klocale.h>

#include "includes.h" // for the misc functions...

#include "misc.h"
#include "myProfile.h"
#include "kPGPShell.h" // for a child widget...

// #include "geheimnis.h"


static const char *description =
    I18N_NOOP("Geheimnis: A PGP and GPG front-end for KDE");

static KCmdLineOptions options[] = {
    { "e", 0, 0 },
    { "encrypt", I18N_NOOP("Encrypt file"), 0 },
    { "s", 0, 0 },
    { "sign", I18N_NOOP("Sign file"), 0 },
    { "d", 0, 0 },
    { "decrypt", I18N_NOOP("Decrypt file"), 0 },
    { "v", 0, 0 },
    { "verify", I18N_NOOP("Verify file"), 0 },
    { "k", 0, 0 },
    { "keys", I18N_NOOP("Open Key Management"), 0 },
    { "i", 0, 0 },
    { "infile <file>", I18N_NOOP("Incoming file"), 0 },
    { "o", 0, 0 },
    { "outfile <file>", I18N_NOOP("Outgoing file"), 0 },
    KCmdLineLastOption
};

const char *license_question = "This program is covered by the GNU Public "
    "License (GPL).\n\nHowever, I want to make sure you understand - there is "
    "NO WARRANTY.\n\nDo you accept this license?";

/* This is used to prevent zombies */
void sigchld_handler(int) {
    wait(0);
}


int main(int argc, char *argv[]) {
    KAboutData aboutData("geheimnis", I18N_NOOP("Geheimnis"),
			 MY_VERSION, description, KAboutData::License_GPL,
			 "(c) 2000, Chris Wiegand");
    // FIXME Chris: Check to make sure we're SUPPOSED to be LGPL!!!
    aboutData.addAuthor("Chris Wiegand",0, "cwiegand@urgentmail.com");
    aboutData.addAuthor("Stefan Suchi",0, "suchi@gmx.de");
    KCmdLineArgs::init(argc, argv, &aboutData);
    KCmdLineArgs::addCmdLineOptions(options); // Add our own options.

    KApplication app;

/* Not Yet Supported FIXME
  if (app.isRestored())
  {
    RESTORE(GeheimnisApp);
  }
  else
  {
*/

    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

    // Before anything else, let's check the license (since we're going GUI anyways...)

    // Okay, first, if we haven't agreed to the license, let's do that...
    KConfig *cfg = kapp->config();
    cfg->setGroup("__Global");
    if (cfg->readEntry("LicenseOK", "").isEmpty()) {
	int r = 0;
	r = KMessageBox::questionYesNo(NULL, i18n(license_question));
	if (r == KMessageBox::Yes) cfg->writeEntry("LicenseOK", "Yes");
	else return -1; // Didn't accept license, sorry buster!
    }

    // Check arguments for the appropriate action (if any)
    int action = 6;
    if (args->isSet("encrypt") || args->isSet("sign"))
	action = 2;
    if (args->isSet("decrypt") || args->isSet("verify")) {
	if (action != 6)
	    KCmdLineArgs::usage(i18n("Don't specify more than one type of "
				     "action"));
	action = 3;
    }
    if (args->isSet("keys")) {
	if (action != 6)
	    KCmdLineArgs::usage(i18n("Don't specify more than one type of "
				     "action"));
	action = 1;
    }
    if (args->isSet("outfile") && !args->isSet("infile"))
	KCmdLineArgs::usage(i18n("Give an input file if you give an output "
				 "file"));

    /* Make sure that zombies don't hold up processes */
    if (signal(SIGCHLD, sigchld_handler) == SIG_ERR) {
	fprintf(stderr, "I'm sorry, but I couldn't start my Zombie Process "
		"Handler. Please see http://geheimnis.sourceforge.net for "
		"assistance.\n");
	return -1;
    }

    checkPrefs();

    QString infile = args->getOption("infile");
    QString outfile = args->getOption("outfile");
    KMainWindow *toplevel = droppedFile(action, infile, outfile);
    if (!toplevel) return -1;
    toplevel->show();
    app.setMainWidget(toplevel);

    return app.exec();
}
