
#include "misc.h"

// I moved these here so as not to create a loop betwixt kPGP* and keyring and misc...
#include "kPGPPrefs.h"
#include "kPGPKeys.h"
#include "kPGPDecrypt.h"
#include "kPGPEncrypt.h"
#include "kPGPDNDZone.h"
#include "kPGPProfileEdit_New.h"
#include "myProfile.h"

// -----------------------------------------------------------------

char *getSafeTmpName() {
	char *tmp1 = NULL;

	tmp1 = (char *) myMalloc(1024);
	if (tmp1) {
		snprintf(tmp1,1023,"%s/geheimnis.XXXXXX",getenv("HOME"));
		mkstemp(tmp1);
	}
	return tmp1;
}

// returns a QString equiv of getSafeTmpName
QString *getSafeTmpNameQ() {
	QString *ret = new QString(""); // just to be safe...
	char *tmp = NULL;
	tmp = getSafeTmpName();
	ret = new QString(tmp);
	if (tmp) free(tmp);
	return ret;
}

char *extractDisplayName(const char *stringIn)
{
	// I return a pointer that needs to be "free"ed!
	char *retVal = NULL;
	QString *tmpString, *tmpString2;
	int s,e;

	tmpString = new QString(stringIn);
	s = tmpString->find("<");
	e = tmpString->find(">");
	if (s > 0 && e > s)
		tmpString2 = new QString(tmpString->mid(0,s-1));
	else
		tmpString2 = new QString(stringIn);

	retVal = (char *) myMalloc(tmpString2->length()+2);
	strcpy(retVal,tmpString2->data());
	return retVal;
}

char *extractEmailAddress(const char *stringIn)
{
	// I return a pointer that needs to be "free"ed!
	char *retVal = NULL;
	QString *tmpString, *tmpString2;
	int s,e;

	tmpString = new QString(stringIn);
	s = tmpString->find("<");
	e = tmpString->find(">");
	if (s > 0 && e > s)
		tmpString2 = new QString(tmpString->mid(s+1,e-s-1));     // second value is length -- Stefan
	else
		tmpString2 = new QString(stringIn);

	retVal = (char *) myMalloc(tmpString2->length()+2);
	strcpy(retVal,tmpString2->data());
	delete tmpString;
	delete tmpString2;
	return retVal;
}

int runProcessFG(const char pgpAppString[MAX_COMMAND_LINE_LENGTH], int keepopen)
{
	QString sRunMe, sCmdLine, sTmp;
	int c, loc;
  KConfig *cfg = NULL;

  cfg = kapp->config();
  if (cfg == NULL) return -1; // failed...
	cfg->setGroup("__Global");

	if (keepopen == 1) sCmdLine += cfg->readEntry("PathKeepopen","geheimniskeepopen");
	//	if (keepopen == 1) sCmdLine += "geheimniskeepopen ";
	sCmdLine += " "; // just to be safe...
	sCmdLine += pgpAppString;

	sRunMe = cfg->readEntry("Terminal","konsole -e %C");

	// Run through finding the %T, if any...
	loc = sRunMe.find("%T",0,FALSE);
	if (loc > -1) {
		sTmp.sprintf("\"%s\"",i18n("Geheimnis").data());
		// remove the location, two chars long, and add title
		sRunMe.remove(loc,2);
		sRunMe.insert(loc,sTmp.data());
	}

	// Now, hash through, finding the %C, if any. If none, append to end..
	loc = sRunMe.find("%C",0,FALSE);
	if (loc > -1) {
		// remove the location, two chars long, then insert at that point...
		sRunMe.remove(loc,2);
		sRunMe.insert(loc,sCmdLine);
	} else {
		sRunMe += sCmdLine;
	}

#ifdef MY_DEBUG
	printf("Trying: %s\n", sRunMe.data()); fflush(stdout);
#endif
	c = system(sRunMe);
#ifdef MY_DEBUG
	printf("Result: %i\n",c);
#endif
	return c;
}

KMainWindow *droppedFile(int action, const QString infile, const QString outfile) {
	KMainWindow *a = NULL;

	if (action == 0 && infile.isEmpty())
	    action = 6; // default with no file.

	if (action == 0) {
		int iStart = infile.findRev('.');
		if (iStart > 0) {
		        // t is used to prevent changing infile
		        QString t = infile.right(infile.length() - iStart).lower();

			if (t.compare("asc") == 0 || t.compare("pgp") == 0)
			    action = 3;
			else if (t.compare("sig") == 0)
			    action = 4;
		}
	}

	if (action == 0) {
		switch (
			KMessageBox::questionYesNo(
				NULL,
				i18n("I can't determine if this file is encrypted or not.\nPlease tell me what you want me to do."),
				i18n("Geheimnis"),
				i18n("Encrypt/Sign file"),
				i18n("Decrypt/Verify file")
			)
		) {
				case KMessageBox::Yes: // encrypt/sign
					action = 2;
					break;
				case KMessageBox::No: // decrypt/verify
					action = 3;
					break;
		}
	}

	switch (action) {
		case 0:
		case 6: // deprecated, technically, but we still support it, it's easier...
			a = new kPGPDNDZone();
			break;

		case 1: // for key management
			a = new kPGPKeys();
			break;

		case 2: // for signing / encrypting files
			a = new kPGPEncrypt(infile, outfile, 0);
			break;

		case 3: // for inline signatures / all encrypted files
			a = new kPGPDecrypt(infile, outfile, 0, false);
			break;

		case 4: // for detached signatures...
			a = new kPGPDecrypt(infile, outfile, 0, true);
			break;
	}
	return a;
}

void killFirstChar(char *inString)
{
 	unsigned int c;
 	// inString is passed back, so it's a pointer
 	for (c=0;c<strlen(inString);c++) inString[c] = inString[c+1];
}

void cpString(FILE *inFile, char buf[65530])
{
	// DO NOT REPLACE ME, GNUPG NEEDS ME FOR BUFFERING... -- chris
	char ch;
	int i = 0;

	ch = fgetc(inFile);

	while((ch != EOF) && (ch != '\n'))
	{
		buf[i] = ch;
		ch = fgetc(inFile);
		i++;
	}
	buf[i] = '\0';
}

int pipe_pgp(const char *cmd, FILE **in, FILE **out)
{
         int pin[2], pout[2], child_pid;

#ifdef MY_DEBUG
	printf("pipe_pgp DEBUG: %s\n",cmd);
#endif
         *in = *out = NULL;

         pipe(pin);
         pipe(pout);

         if(!(child_pid = fork()))
         {
           // We're the child.
           close(pin[1]);
           dup2(pin[0], 0);
           close(pin[0]);

           close(pout[0]);
           dup2(pout[1], 1);
           close(pout[1]);

           execl("/bin/sh", "sh", "-c", cmd, NULL);
           _exit(127);
         }

         // Only get here if we're the parent.
         close(pout[1]);
         *out = fdopen(pout[0], "r");

         close(pin[0]);
         *in = fdopen(pin[1], "w");

         return(child_pid);
}

void *myMalloc(size_t size)
{
	void *newPtr;
	newPtr = malloc(size);
	if (!newPtr) {
		fprintf(stderr,"Fatal Error: not able to malloc (%d bytes)!\n",size);
		exit(-2);
	}
	return newPtr;
}

int getItemCount(const char *incoming, const char delimiter)
{
	int j = 1;
	uint i = 0;
	for (i=0;i<strlen(incoming);i++) if (incoming[i] == delimiter) j++;
	return j;
}

char *getItem(const char *incoming, const char delimiter, unsigned int fieldNum)
{
	// I support a string as long as an "unsigned long" is big,
	//     and as many fields as an "unsigned int" is big.
	// I return a pointer that must be "free"ed!

	char *tmpString = NULL, *retVal=NULL, *tmpStr = NULL;
	unsigned long l;
	unsigned int thisItem=1;

	tmpString = (char *) myMalloc(strlen(incoming)+2);
	retVal = (char *) myMalloc(strlen(incoming)+2);    // 2 just to be safe.
	strcpy(tmpString,incoming); // so I can modify it... don't need strcpy, it's malloced the strlen(incoming)+2
	strcpy(retVal,"");

	// I only set startChar and endChar on a delimiter so that if the
	// requested item is the 1st item, it starts and ends on item #1.

	for (l=0;l<=strlen(tmpString);l++) {
		if (tmpString[l] == delimiter) {
			thisItem++;
		} else {
			if (thisItem == fieldNum) {
				tmpStr = tmpString+l; // we want to set the "tmpStr" to a particular starting place within tmpString;
				strncat(retVal,tmpStr,1); // now, copy 1 ONE character...
			}
		}
	}

	if (tmpString != NULL) free(tmpString);
	return retVal;
}

void checkPrefs()
{
	/* Note: this function is called from main.c, kPGPKeys.cpp, and
	 * possibly others to verify that we have at least one profile...
	 */
	KConfig *cfg = NULL;
	QString a;
	kPGPPrefs *pe = NULL;

	cfg = kapp->config();
	cfg->setGroup("__Global");

	QStrList profileList;

	// Now we add anything new...
	cfg->readListEntry("ProfileList",profileList,';');
	if (profileList.count() == 0) {
		// addDefaultProfiles(cfg);
		KMessageBox::information(NULL,i18n("Welcome to Geheimnis! We will now setup a profile for\nuse with Geheimnis. You can configure as many profiles\nas you want. Usually, most people configure one profile\nfor each encryption program they have installed.\n\nClick OK to configure your first profile."),i18n("Geheimnis"));
		if (!pe) pe = new kPGPPrefs();
		pe->show();
	}

	cfg->setGroup("__Global");
	// Okay, we check to see if we have a PathKeepopen.
	// If so, check to make sure it's a real file
	// If not, prompt them to set one!
	a = cfg->readEntry("PathKeepopen","");
	if (strlen(a.data()) < 5) {
		KMessageBox::information(NULL,i18n("The path to geheimniskeepopen has not been set yet.\nPlease find that program now.\nIt is usually located in /usr/bin or /usr/local/bin,\nunless set somewhere else by your administrator."));
		if (!pe) pe = new kPGPPrefs();
		pe->show();
	} else {
		QFileInfo *qfi = NULL;
		qfi = new QFileInfo(a.data());
		if (!qfi->isExecutable()) {
			KMessageBox::sorry(NULL,i18n("The path to geheimniskeepopen has changed.\nPlease find that program now.\nIt is usually located in /usr/bin or /usr/local/bin,\nunless set somewhere else by your administrator."));
			if (!pe) pe = new kPGPPrefs();
			pe->show();
		}
	}

	// Write our revision for future upgrades...
	cfg->setGroup("__Global");
	if (cfg->readNumEntry("Revision",0) < MY_REVISION) {
		// New revision!
		if (KMessageBox::questionYesNo(NULL,i18n("This is the first time you're starting this version of Geheimnis.\nDo you want to look at the ChangeLog?"),i18n("New Revision")) == KMessageBox::Yes)
			kapp->invokeHTMLHelp("geheimnis/changelog.html","");
			// display changelog...
	}

	cfg->writeEntry("Revision",MY_REVISION);

	cfg->sync();
	return;
}

QStrList *getProfileList()
{
	// I return a pointer that must be "free"ed!
	KConfig *cfg;
	cfg = kapp->config();
	if (cfg == NULL) return NULL; // failed

	QStrList *newList;
	newList = new QStrList;
	cfg->setGroup("__Global");
	cfg->readListEntry("ProfileList",*newList,';');
	return newList;
}

void addDefaultProfiles(KConfig *cfg)
{
	KMessageBox::information(NULL,i18n("Please choose Profile Setup from the File menu\nafter I setup your initial profiles to remove\nany unnecessary profiles."));

	cfg->setGroup("GnuPG 1.0x");
	cfg->writeEntry("PGPVersion","1");

	cfg->setGroup("PGP 5.0");
	cfg->writeEntry("PGPVersion","5");

	cfg->setGroup("PGP 2.6");
	cfg->writeEntry("PGPVersion","2");

	cfg->setGroup("PGP 6.5");
	cfg->writeEntry("PGPVersion","6");

	QStrList tmp;
	tmp.append("GnuPG 1.0x");
	tmp.append("PGP 5.0");
	tmp.append("PGP 2.6");
	tmp.append("PGP 6.5");

	cfg->setGroup("__Global");
	cfg->writeEntry("ProfileList",tmp,';');

	cfg->sync();
}

void setToDefaultProfile(QComboBox *cb)
{
	int i = 0, j = 0;
	QString a;
	KConfig *cfg = NULL;

	cfg = kapp->config();
	cfg->setGroup("__Global");
	a = cfg->readEntry("DefaultProfile","");
	if (a.length() > 1) {
		for (i=0;i<cb->count();i++) {
			if (a == cb->text(i)) j = i;
// FIXME		printf("j = %i    i = %i   a = %s  cb(j) = %s   cb(i) = %s\n",j,i,a.data(),cb->text(j),cb->text(i));
		}
	}
	cb->setCurrentItem(j);
}

void incrQPD(QProgressDialog *qpd)
{
	qpd->setProgress(qpd->progress() + 1);
	if (qpd->progress() >= qpd->totalSteps()) qpd->setProgress(0);
	qApp->processEvents();
}

QString *convertQStrListToQString(QStrList *a)
{
	QString *s = new QString("");
	uint i = 0;

	if (a) {
		for (i=0;i<a->count();i++) {
			s->append(a->at(i));
			s->append(" ");
		}
	}
	return s;
}

void showBadProfileDialog() {
	KMessageBox::error(NULL,i18n("There was an error with the profile you selected. Please select another, or close and re-open this window."));
}

void showBadKeyIDDialog() {
	KMessageBox::error(NULL,i18n("You must select a key to do that!"));
}

void showUnsupportedDialog() {
	KMessageBox::sorry(NULL,i18n("Sorry, but your encryption program does not support that feature."));
}

