
#include "myKeyring.h"

/*
 * --------------------------------- pkiSig ------------------------------
 */

pkiSig::pkiSig(pkiName *theName, const char *signKeyID)
{
	signingKeyID = "";
	if (signKeyID) signingKeyID = signKeyID;
	parentName = theName;
	parentSubKey = NULL;
	theName->sigs.append(this);
	return;
}

pkiSig::pkiSig(pkiSubKey *theSK, const char *signKeyID)
{
	signingKeyID = "";
	if (signKeyID) signingKeyID = signKeyID;
	parentName = NULL;
	parentSubKey = theSK;
	theSK->sigs.append(this);
	return;
}

pkiName *pkiSig::getParentName()
{
	return parentName;
}

pkiSubKey *pkiSig::getParentSubKey()
{
	return parentSubKey;
}

QString *pkiSig::getSigningKeyID()
{
	QString *a = new QString(signingKeyID);
	return a;
}

pkiKey *pkiSig::getSigningKey()
{
	myProfile *a;
	pkiKey *b;
	// Have to find the profile, so we go up the chain...
	if (parentName) {
		b = parentName->getParentKey();
	} else {
		b = parentSubKey->getParentKey();
	}
	a = b->parentProfile;
	return a->getKey(signingKeyID.data());
}

/*
 * --------------------------------- pkiName ------------------------------
 */

pkiName::pkiName(pkiKey *theKey, const char *dName, const char *eMail) 
{
	displayName = "";
	if (dName) displayName = dName;
	emailAddress = "";
	if (eMail) emailAddress = eMail;
	parentKey = theKey;
	if (dName && !theKey->names.isEmpty()) { // FIXME does this even work?
		// Okay, we'll see if perhaps we shouldn't place this at the end...
		if (strncmp(theKey->names.at(0)->getDisplayName()->data(),"*",1) == 0) {
			theKey->names.insert(0,this);     // garbage in first name, so replace
		} else if (strncmp(theKey->names.at(0)->getDisplayName()->data(),"[",1) == 0) {
			theKey->names.insert(0,this);     // garbage in first name, so replace
		} else
			theKey->names.append(this);
	} else
		theKey->names.append(this);
	return;
}

pkiKey *pkiName::getParentKey()
{
	return parentKey;
}

QString *pkiName::getDisplayName() 
{
	QString *a = new QString(displayName);
	return a;
}

QString *pkiName::getEmailAddress()
{
	QString *a = new QString(emailAddress);
	return a;
}

/*
 * --------------------------------- pkiSubKey ----------------------------
 */

pkiSubKey::pkiSubKey(pkiKey *theKey, const char *newSKeyID)
{
	keyID = "";
	if (newSKeyID) keyID = newSKeyID;
	keySize = "0";
	keyType = "Unknown";
	expireDate = "--------";
	createDate = "--------";
	parentKey = theKey;
	theKey->subkeys.append(this);
	return;
}

pkiKey *pkiSubKey::getParentKey()
{
	return parentKey;
}

QString *pkiSubKey::getKeyID()
{
	QString *a = new QString(keyID);
	return a;
}

void pkiSubKey::setKeySize(QString *a)
{
	keySize = a->data(); // I want hard copies! -- chris
}

void pkiSubKey::setKeySize(const char *a)
{
	keySize = a;
}

QString *pkiSubKey::getKeySize()
{
	QString *a = new QString(keySize);
	return a;
}

void pkiSubKey::setKeyType(QString *a)
{
	keyType = a->data();
}

void pkiSubKey::setKeyType(const char *a)
{
	keyType = a;
}

QString *pkiSubKey::getKeyType()
{
	QString *a = new QString(keyType);
	return a;
}

void pkiSubKey::setExpireDate(QString *a)
{
	expireDate = a->data(); // HARD copy
}

void pkiSubKey::setExpireDate(const char *a)
{
	expireDate = a;
}

QString *pkiSubKey::getExpireDate()
{
	QString *a = new QString(expireDate);
	return a;
}

void pkiSubKey::setCreateDate(QString *a)
{
	createDate = a->data(); // HARD copy
}

void pkiSubKey::setCreateDate(const char *a)
{
	createDate = a; 
}

QString *pkiSubKey::getCreateDate()
{
	QString *a = new QString(createDate);
	return a;
}

/*
 * --------------------------------- pkiKey ------------------------------
 */

pkiKey::pkiKey(myProfile *profile, const char *newKeyID)
{
	// defaults
	keyID = "";
	if (newKeyID) keyID = newKeyID;
	keySize = "0";
	keyType = "Unknown";
	expireDate = "--------";
	createDate = "--------";
	haveSecret = FALSE;
	isExpired = 0;
	isRevoked = 0;
	isDisabled = 0;
	validity = 0;
	trust = 0;
	parentProfile = profile;
	profile->keyring.append(this); // now we're double-linked...
	return;
}

QString *pkiKey::getKeyID()
{
	QString *a = new QString(keyID);
	return a;
}

QString *pkiKey::getKeySize()
{
	QString *a = new QString(keySize);
	return a;
}

void pkiKey::setKeySize(QString *a)
{
	keySize = a->data(); // I want HARD copies -- chris
}

void pkiKey::setKeySize(const char *a)
{
	keySize = a;
}

QString *pkiKey::getKeyType()
{
	QString *a = new QString(keyType);
	return a;
}

void pkiKey::setKeyType(QString *a)
{
	keyType = a->data();
}

void pkiKey::setKeyType(const char *a)
{
	keyType = a;
}

QString *pkiKey::getFingerprint()
{
	QString *a = new QString(fingerprint);
	return a;
}

void pkiKey::setFingerprint(QString *a)
{
	fingerprint = a->data();
}

void pkiKey::setFingerprint(const char *a)
{
	fingerprint = a;
}

QString *pkiKey::getExpireDate()
{
	QString *a = new QString(expireDate);
	return a;
}

void pkiKey::setExpireDate(QString *a)
{
	expireDate = a->data();
}

void pkiKey::setExpireDate(const char *a)
{
	expireDate = a;
}

QString *pkiKey::getCreateDate()
{
	QString *a = new QString(createDate);
	return a;
}

void pkiKey::setCreateDate(QString *a)
{
	createDate = a->data();
}

void pkiKey::setCreateDate(const char *a)
{
	createDate = a;
}

int pkiKey::getValidity()
{
	return validity;
}

void pkiKey::setValidity(int i)
{
	validity = i;
}

int pkiKey::getTrust()
{
	return trust;
}

void pkiKey::setTrust(int i)
{
	trust = i;
}


