
#ifndef GEHEIMNIS_MYKEYRING_H
#define GEHEIMNIS_MYKEYRING_H

#include "includes.h"
#include "defines.h"

class pkiSig {
	public:
		pkiSig(pkiName *theName, const char *signKeyID=NULL);
		pkiSig(pkiSubKey *theSK, const char *signKeyID=NULL);

		pkiName *getParentName();
		pkiSubKey *getParentSubKey();
		QString *getSigningKeyID();
		pkiKey *getSigningKey();

	private:		
		pkiName *parentName; // One of these WILL be null...
		pkiSubKey *parentSubKey;
		QString signingKeyID;
};

class pkiName {
	public:
		pkiName(pkiKey *theKey, const char *dName=NULL, const char *eMail=NULL);

		pkiKey *getParentKey();
		QString *getDisplayName();
		QString *getEmailAddress();
		QList<pkiSig> sigs;
		
	private:
		pkiKey *parentKey;
		QString emailAddress;
		QString displayName;
};

class pkiSubKey {
	public:
		pkiSubKey(pkiKey *theKey, const char *newSKeyID=NULL);

		pkiKey *getParentKey();
		QString *getKeyID(); // must be set in constructor
		QString *getKeySize();
		void setKeySize(QString *a);
		void setKeySize(const char *a);
		QString *getKeyType();
		void setKeyType(QString *a);
		void setKeyType(const char *a);
		QString *getExpireDate();
		void setExpireDate(QString *a);
		void setExpireDate(const char *a);
		QString *getCreateDate(); // FIXME: QDate??
		void setCreateDate(QString *a);
		void setCreateDate(const char *a);
		QList<pkiSig> sigs;

	private:		
		pkiKey *parentKey;
		QString expireDate;
		QString createDate;
		QString keyID;
		QString keySize;
		QString keyType;
};

class pkiKey {
	public:
		pkiKey(myProfile *profile, const char *newKeyID=NULL); // set defaults

		myProfile *parentProfile;
		QString *getKeyID();
		QString *getKeySize();
		void setKeySize(QString *a);
		void setKeySize(const char *a);
		QString *getKeyType();
		void setKeyType(QString *a);
		void setKeyType(const char *a);
		QString *getFingerprint();
		void setFingerprint(QString *a);
		void setFingerprint(const char *a);
		QList<pkiName> names;
		QList<pkiSubKey> subkeys;
		bool haveSecret;
		bool isRevoked;
		bool isExpired;
		bool isDisabled;
		QString *getExpireDate();
		void setExpireDate(QString *a);
		void setExpireDate(const char *a);
		QString *getCreateDate();
		void setCreateDate(QString *a);
		void setCreateDate(const char *a);
		int getValidity();
		void setValidity(int i);
		int getTrust();
		void setTrust(int i);

	private:
		int trust;      // How much do we trust keys he's signed?
		int validity;   // Is this key signed by people we trust?
		QString expireDate; // DEPRECATE FIXME
		QString createDate; // DEPRECATE FIXME
		QString keyType;
		QString keyID;
		QString keySize;
		QString fingerprint;

};

// -----------------------------------------------------------------

/* Trust/Validity Values:
 * 0 = Unknown/Not Assigned
 * 1 = Not Trusted/Valid
 * 2 = Marginally Trusted/Valid
 * 3 = Fully Trusted/Valid
 * 4 = Implicit/Ultimate Trust/Validity
 */

// -----------------------------------------------------------------
 
#include "myProfile.h"

#endif
