
#include "myProfile.h"
#include "misc.h"

myProfile::myProfile(int newPGPVersion, const char *folPath)
{
	KConfig *cfg = NULL;
	QString sTmp;
	char *homeEnv = NULL;

	homeEnv = getenv("HOME");

	cfg = kapp->config();
	if (cfg == NULL) return; // failed...

	// was pgpVersion, not newPGPVersion
	switch (newPGPVersion) {
		case 6:
			cfg->setGroup(i18n(PGP6_NAME));
			profileName = i18n(PGP6_NAME);
			cfg->writeEntry("PGPVersion",6);
			sTmp.sprintf("%s%s",folPath,"pgp");
			cfg->writeEntry("binPGP6",sTmp.data());
			sTmp.sprintf("%s/.pgp/secring.skr",homeEnv);
			cfg->writeEntry("SecretKeyring",sTmp.data());
			sTmp.sprintf("%s/.pgp/pubring.pkr",homeEnv);
			cfg->writeEntry("PublicKeyring",sTmp.data());
			cfg->writeEntry("Keyserver","search.keyserver.net");
			break;
		case 5:
			cfg->setGroup(i18n(PGP5_NAME));
			profileName = i18n(PGP5_NAME);
			sTmp.sprintf("%s%s",folPath,"pgpk");
			cfg->writeEntry("binPGP5K",sTmp.data());
			sTmp.sprintf("%s%s",folPath,"pgpe");
			cfg->writeEntry("binPGP5E",sTmp.data());
			sTmp.sprintf("%s%s",folPath,"pgps");
			cfg->writeEntry("binPGP5S",sTmp.data());
			sTmp.sprintf("%s%s",folPath,"pgpv");
			cfg->writeEntry("binPGP5V",sTmp.data());
			sTmp.sprintf("%s/.pgp/secring.skr",homeEnv);
			cfg->writeEntry("SecretKeyring",sTmp.data());
			sTmp.sprintf("%s/.pgp/pubring.pkr",homeEnv);
			cfg->writeEntry("PublicKeyring",sTmp.data());
			break;
		case 2:
			cfg->setGroup(i18n(PGP2_NAME));
			profileName = i18n(PGP2_NAME);
			sTmp.sprintf("%s/pgp",folPath);
			cfg->writeEntry("binPGP2",sTmp.data());
			sTmp.sprintf("%s/.pgp/secring.pgp",homeEnv);
			cfg->writeEntry("SecretKeyring",sTmp.data());
			sTmp.sprintf("%s/.pgp/pubring.pgp",homeEnv);
			cfg->writeEntry("PublicKeyring",sTmp.data());
			break;
		case 1:
			cfg->setGroup(i18n(GPG_NAME));
			profileName = i18n(GPG_NAME);
			sTmp.sprintf("%s/gpg",folPath);
			cfg->writeEntry("binGPG",sTmp.data());
			sTmp.sprintf("%s/.gnupg/secring.gpg",homeEnv);
			cfg->writeEntry("SecretKeyring",sTmp.data());
			sTmp.sprintf("%s/.gnupg/pubring.gpg",homeEnv);
			cfg->writeEntry("PublicKeyring",sTmp.data());
			cfg->writeEntry("Keyserver","search.keyserver.net");
			break;
	}

	cfg->sync();
	loadProfile(profileName.data()); // load that which we just created...
}


myProfile::myProfile(const char *newProfileName)
{
	loadProfile(newProfileName);
}

void myProfile::loadProfile(const char *newProfileName)
{
	KConfig *cfg = NULL;
	uint i=0;
	QString sTmp;
	char *homeEnv = NULL;

	homeEnv = getenv("HOME");

	cfg = kapp->config();
	if (cfg == NULL) return; // failed...

	cfg->setGroup(newProfileName);
	profileName.sprintf("%s",newProfileName); // Have to use sprintf to avoid const char warnings...
	pgpVersion = cfg->readNumEntry("PGPVersion",0);

	keyServer = "";

	switch (pgpVersion) {
		case 6:
			if (strlen(cfg->readEntry("binPGP6","")) < 2) cfg->writeEntry("binPGP6","pgp");
			pgpExecs.append(cfg->readEntry("binPGP6","pgp"));
			if (strlen(cfg->readEntry("SecretKeyring","")) < 2) cfg->writeEntry("SecretKeyring","~/.pgp/secring.skr");
			if (strlen(cfg->readEntry("PublicKeyring","")) < 2) cfg->writeEntry("PublicKeyring","~/.pgp/pubring.pkr");
			keyServerCompatible = TRUE;
			if (strlen(cfg->readEntry("Keyserver","")) < 2) cfg->writeEntry("Keyserver","search.keyserver.net");
			keyServer = cfg->readEntry("Keyserver","search.keyserver.net");
			recipFormat = " %s ";
			break;
		case 5:
			if (strlen(cfg->readEntry("binPGP5K","")) < 2) cfg->writeEntry("binPGP5K","pgpk");
			pgpExecs.append(cfg->readEntry("binPGP5K","pgpk"));
			if (strlen(cfg->readEntry("binPGP5E","")) < 2) cfg->writeEntry("binPGP5E","pgpe");
			pgpExecs.append(cfg->readEntry("binPGP5E","pgpe"));
			if (strlen(cfg->readEntry("binPGP5S","")) < 2) cfg->writeEntry("binPGP5S","pgps");
			pgpExecs.append(cfg->readEntry("binPGP5S","pgps"));
			if (strlen(cfg->readEntry("binPGP5V","")) < 2) cfg->writeEntry("binPGP5V","pgpv");
			pgpExecs.append(cfg->readEntry("binPGP5V","pgpv"));
			if (strlen(cfg->readEntry("SecretKeyring","")) < 2) cfg->writeEntry("SecretKeyring","~/.pgp/secring.skr");
			if (strlen(cfg->readEntry("PublicKeyring","")) < 2) cfg->writeEntry("PublicKeyring","~/.pgp/pubring.pkr");
			keyServerCompatible = TRUE;
			recipFormat = " -r %s ";
			break;
		case 2:
			if (strlen(cfg->readEntry("binPGP2","")) < 2) cfg->writeEntry("binPGP2","pgp");
			pgpExecs.append(cfg->readEntry("binPGP2","pgp"));
			if (strlen(cfg->readEntry("SecretKeyring","")) < 2) cfg->writeEntry("SecretKeyring","~/.pgp/secring.pgp");
			if (strlen(cfg->readEntry("PublicKeyring","")) < 2) cfg->writeEntry("PublicKeyring","~/.pgp/pubring.pgp");
			keyServerCompatible = FALSE;
			recipFormat = " %s ";
			break;
		case 1:
			if (strlen(cfg->readEntry("binGPG","")) < 2) cfg->writeEntry("binGPG","gpg");
			pgpExecs.append(cfg->readEntry("binGPG","gpg"));
			if (strlen(cfg->readEntry("SecretKeyring","")) < 2) cfg->writeEntry("SecretKeyring","~/.gnupg/secring.gpg");
			if (strlen(cfg->readEntry("PublicKeyring","")) < 2) cfg->writeEntry("PublicKeyring","~/.gnupg/pubring.gpg");
			keyServerCompatible = TRUE;
			if (strlen(cfg->readEntry("Keyserver","")) < 2) cfg->writeEntry("Keyserver","search.keyserver.net");
			keyServer = cfg->readEntry("Keyserver","search.keyserver.net");
			recipFormat = " -r %s ";
			break;
	}

	cfg->readListEntry("SecretKeyring",secretKeyringFiles,';');
	cfg->readListEntry("PublicKeyring",publicKeyringFiles,';');

	// Now, scan lists for ~, replace with getenv("HOME");
	for (i=0;i<secretKeyringFiles.count();i++) {
		sTmp = secretKeyringFiles.at(i);
		if (strcmp(sTmp.left(1),"~") == 0) {
			sTmp.remove(0,1);
			sTmp.prepend(homeEnv);
			secretKeyringFiles.remove(i);
			secretKeyringFiles.insert(i,sTmp.data());
		}
	}
	for (i=0;i<publicKeyringFiles.count();i++) {
		sTmp = publicKeyringFiles.at(i);
		if (strcmp(sTmp.left(1),"~") == 0) {
			sTmp.remove(0,1);
			sTmp.prepend(homeEnv);
			publicKeyringFiles.remove(i);
			publicKeyringFiles.insert(i,sTmp.data());
		}
	}

	keyServer = cfg->readEntry("Keyserver","pgpkeys.mit.edu");
	cfg->sync();
	// FIXME: research - do we need to free homeEnv?
	return;
}

myProfile::~myProfile() {
}

QString myProfile::keyringsToUse(const bool bIncludeSecret, const bool bIncludePublic)
{
	// I use QString because it auto-sizes, unlike char *...
	QString pgpAppString("");
	unsigned int i; // general counter...

	switch (pgpVersion) {
		case 1:
			pgpAppString.sprintf(" --no-default-keyring ");
			if (bIncludeSecret) {
				for (i=0;i<secretKeyringFiles.count();i++) {
					pgpAppString.append(" --secret-keyring ");
					pgpAppString.append(secretKeyringFiles.at(i));
				}
			}
			if (bIncludePublic) {
				for (i=0;i<publicKeyringFiles.count();i++) {
					pgpAppString.append(" --keyring ");
					pgpAppString.append(publicKeyringFiles.at(i));
				}
			}
			break;
		case 2:
			// PGP 2 doesn't support keyring specified in command line.
			break;
		case 5:
			if (bIncludeSecret) {
				if (bIncludePublic) {
					pgpAppString.sprintf(" --secring=%s --pubring=%s",secretKeyringFiles.at(0),publicKeyringFiles.at(0));
				} else {
					pgpAppString.sprintf(" --secring=%s",secretKeyringFiles.at(0));
				}
			} else {
				if (bIncludePublic) {
					pgpAppString.sprintf(" --pubring=%s",publicKeyringFiles.at(0));
				}
			}
			break;
		case 6:
 			// for me it looks like PGP-6.5 doesn't support keyring specified in command line.
 			// maybe I'm too stupid, but --secring=%s is definitely not working  -- stefan
			break;
	}
	return pgpAppString;
}

// -----------------------------------------------------------------

void myProfile::loadKeyring(const bool justRefreshTrust, const char *keyID)
{
	/* DO NOT DELETE newKey, tmpKey, newName, or newSig under punishment of death!
	 * That would corrupt the keyring, which is what we're returning!
	 *
	 * This function is VERY GENERIC and just calls sub-functions to do all
	 * of the actual work...
	 *
	 * justRefreshTrust just refreshes the validity/trust vales, does not load new keys fully...
	 * if you specify a keyID, justRefreshTrust is ignored...
	 * keyID , if not null, will only load that key, but fully, no matter what justRefreshTrust says
	 * if it's null, it loads the whole keyring...
	 */

	QString appString;
	QProgressDialog *winProgress = new QProgressDialog(i18n("Loading keyring..."),0,200);

	if (!justRefreshTrust && keyID == NULL) keyring.clear();

	winProgress->setProgress(0);
	winProgress->show();
	qApp->processEvents();

	if (pgpVersion == 0) return;

	// Programmer's note: When scanning, **ALWAYS** check before adding a key, even if you only load the ring once.
	// This way, I will always be sure duplicates don't exist and everything links correctly.
	// Programmer's note 2: ONLY LOAD NAMES ONCE AND ONLY ONCE!!!!! -- chris and stefan

	// Modes are not globally defined - you can have as many modes as you want, please define at top of function

	qApp->processEvents();
	switch (pgpVersion) {
 		case 6:
			if (keyID) {
	 			appString.sprintf("%s +LANGUAGE=en -kvv %s", pgpExecs.at(0),keyID);
	 			loadPGP6Keyring(appString.data(),2, winProgress);
	 			appString.sprintf("%s +LANGUAGE=en -kc %s", pgpExecs.at(0),keyID);
	 			loadPGP65Validity(appString.data(), winProgress);		
			} else {
				if (justRefreshTrust) {
					// just load validity again...
					appString.sprintf("%s +LANGUAGE=en -kc", pgpExecs.at(0));
		 			loadPGP65Validity(appString.data(), winProgress);		
				} else {
		 			// PGP 6, requires four passes, once for secret keys, then for public keys, then for fingerprints...
		 			// and finally for the validity/trust.
		 			// first thing, pipe the secret, then the public
		 			// we have to give an empty user string to get all users in secring -- stefan
		 			appString.sprintf("%s +LANGUAGE=en -kvv \"\" ~/.pgp/secring.skr",pgpExecs.at(0));
		 			loadPGP6Keyring(appString.data(),1, winProgress);
		 			appString.sprintf("%s +LANGUAGE=en -kvv", pgpExecs.at(0));
		 			loadPGP6Keyring(appString.data(),2, winProgress);
		 			appString.sprintf("%s +LANGUAGE=en -kvc", pgpExecs.at(0));
		 			loadPGP6Keyring(appString.data(),3, winProgress);
		 			appString.sprintf("%s +LANGUAGE=en -kc", pgpExecs.at(0));
		 			loadPGP65Validity(appString.data(), winProgress);		
				}
			}
 			break;
		case 5:
			if (keyID) {
				appString.sprintf("%s +language=us -ll %s %s",pgpExecs.at(0), keyID, keyringsToUse().data());
				loadPGP5Keyring(appString.data(),winProgress);
				appString.sprintf("%s +language=us -c %s %s",pgpExecs.at(0), keyID, keyringsToUse().data());
				loadPGP65Validity(appString.data(), winProgress);
			} else {
				if (justRefreshTrust) {
					appString.sprintf("%s +language=us -c %s",pgpExecs.at(0), keyringsToUse().data());
					loadPGP65Validity(appString.data(), winProgress);
				} else {
					appString.sprintf("%s +language=us -ll %s",pgpExecs.at(0), keyringsToUse().data());
					loadPGP5Keyring(appString.data(),winProgress);
					// sorry, need a second pass to read validity and trust -- stefan
					appString.sprintf("%s +language=us -c %s",pgpExecs.at(0), keyringsToUse().data());
					loadPGP65Validity(appString.data(), winProgress);
				}
			}
			break;
		case 2:
			if (keyID) {
				appString.sprintf("%s +LANGUAGE=en -kvv %s", pgpExecs.at(0),keyID);
				loadPGP2Keyring(appString.data(),2, winProgress);
				appString.sprintf("%s +LANGUAGE=en -kc %s", pgpExecs.at(0),keyID);
				loadPGP2Validity(appString.data(), winProgress);			
			} else {
				if (justRefreshTrust) {
					appString.sprintf("%s +LANGUAGE=en -kc", pgpExecs.at(0));
					loadPGP2Validity(appString.data(), winProgress);			
				} else {
					// PGP 2, requires four passes, once for secret keys, then for public keys, then for fingerprints...
					// and finally for the validity/trust.
					// first thing, pipe the secret, then the public
					appString.sprintf("%s +LANGUAGE=en -kv ~/.pgp/secring.pgp",pgpExecs.at(0));
					loadPGP2Keyring(appString.data(),1, winProgress);
					appString.sprintf("%s +LANGUAGE=en -kvv", pgpExecs.at(0));
					loadPGP2Keyring(appString.data(),2, winProgress);
					appString.sprintf("%s +LANGUAGE=en -kvc", pgpExecs.at(0));
					loadPGP2Keyring(appString.data(),3, winProgress);
					appString.sprintf("%s +LANGUAGE=en +batchmode -kc", pgpExecs.at(0));
					loadPGP2Validity(appString.data(), winProgress);			
				}
			}
			break;
		case 1:
			// Load all keyrings - gpg supports multiples :)
			if (keyID) {
				appString.sprintf("%s --list-sigs %s --with-colons --no-greeting %s",pgpExecs.at(0),keyID,keyringsToUse(TRUE,TRUE).data());
				loadGPGKeyring(appString.data(),1, winProgress);
			} else {
				if (justRefreshTrust) {
					appString.sprintf("%s --list-keys --with-colons --no-greeting %s",pgpExecs.at(0),keyringsToUse(TRUE,TRUE).data());
					loadGPGKeyring(appString.data(),1, winProgress);
				} else {
					appString.sprintf("%s --list-sigs --with-colons --no-greeting %s",pgpExecs.at(0),keyringsToUse(TRUE,TRUE).data());
					loadGPGKeyring(appString.data(),1, winProgress);
					appString.sprintf("%s --fingerprint --with-colons --no-greeting %s",pgpExecs.at(0),keyringsToUse(FALSE,TRUE).data());
					loadGPGKeyring(appString.data(),3, winProgress);
					appString.sprintf("%s --list-secret-keys --with-colons --no-greeting %s",pgpExecs.at(0),keyringsToUse(TRUE,FALSE).data());
					loadGPGKeyring(appString.data(),2, winProgress);
				}
			}
			break;
	}

	winProgress->hide();
	delete winProgress;
	qApp->processEvents();
	return;
}

void myProfile::loadGPGKeyring(const char *appString, int mode, QProgressDialog *winProgress)
{
	// This functions loads each record (line) and sends it to parseGPGKey for merging...
	pkiKey *newKey = NULL;
	pkiSubKey *newSubKey = NULL;
	pkiName *newName = NULL;
	pkiSig *newSig = NULL;
	FILE *inFile, *outFile; // DO NOT FREE ME (I don't think so, that is. )
	char buf[65530], keyType[KEY_TYPE_LENGTH], c[2]; // FIXME: Un-hard code these if possible.
	QString tmpString;
	int inSec=0, lineType=0, inSubKey = 0;

	fprintf(stderr,"%s\n",appString);
	pipe_pgp(appString,&inFile,&outFile);
	qApp->processEvents();
	if (outFile == NULL) return;

	while (!feof(outFile)) {
		incrQPD(winProgress);
		qApp->processEvents();
		cpString(outFile,buf);

#ifdef MY_DEBUG
printf("GPG In: %s\n",buf);
#endif

		// using fgets() causes problems...
		if (strchr(buf,':') != NULL) {
			strncpy(keyType,buf,3);
			lineType = 0;
			if (strncmp(keyType,"sec",3) == 0) lineType = 1; // secret line...
			if (strncmp(keyType,"pub",3) == 0) lineType = 2; // public key
			if (strncmp(keyType,"uid",3) == 0) lineType = 4; // user id
			if (strncmp(keyType,"sig",3) == 0) lineType = 5; // signature...
			if (strncmp(keyType,"fpr",3) == 0) lineType = 7; // fingerprint...
			if (strncmp(keyType,"sub",3) == 0) lineType = 6; // subkey...
			if (strncmp(keyType,"ssb",3) == 0) lineType = 6; // subkey secret...
			if (strncmp(keyType,"rev",3) == 0) lineType = 8; // revokation...

			if (lineType == 1) inSec = 1;
			if (lineType == 2) inSec = 0;

			switch (lineType) {
				case 1:
				case 2:
					// What we do here: scan for the keyID. If we find it, use that key. If not, new key...
					tmpString.sprintf("0%s",getItem(buf,':',5));
					//
					//	Any GPG keyid starting w/ a letter
				        //	must be prefixed with '0' to work.
				        //
				        //	As the additional '0' does no harm
				        //	in case of the keyid starting w/ a
				        //	digit, add it anywhere.
				        //
				        //	Keyid len is 64bits, which makes for
				        //	16 digits, size of array is big enough.
				        //
					newKey = getKey(tmpString.data());
					if (newKey == NULL) {
#ifdef MY_DEBUG
printf("New Key!\n");
#endif
						newKey = new pkiKey(this,tmpString.data());
						//REMOVEFIXME strcpy(newKey->keyID,tmpString.data());
					}
					if (lineType == 1) newKey->haveSecret = TRUE;

					if (mode == 3) break;
					// we dont want the following information if we only read fingerprints -- stefan

					tmpString = getItem(buf,':',2);
					strncpy(c,tmpString.data(),1);
					switch (c[0]) {
						case 'o':
						case 'q':
							newKey->setValidity(0);
							break;
						case 'e':
							newKey->setValidity(0);
							newKey->isExpired = TRUE;
							break;
						case 'd':
							newKey->setValidity(0);
							newKey->isDisabled = 1;
							break;
						case 'n':
							newKey->setValidity(1);
							break;
						case 'm':
							newKey->setValidity(2);
							break;
						case 'f':
							newKey->setValidity(3);
							break;
						case 'u':
							newKey->setValidity(4);
							break;
					}

					// expiration date...
					tmpString = getItem(buf,':',6);
					newKey->setCreateDate(tmpString);
					tmpString = getItem(buf,':',7);
					if (tmpString.length() > 0) newKey->setExpireDate(tmpString);

					// next, key size...
					tmpString = getItem(buf,':',3);
					newKey->setKeySize(tmpString);

					// next, GPG key type...
					tmpString = getItem(buf,':',4);
					if (tmpString == "1") newKey->setKeyType("RSA");
					if (tmpString == "16") newKey->setKeyType("ElGamal");
					if (tmpString == "17") newKey->setKeyType("DSA");
					if (tmpString == "20") newKey->setKeyType("ElGamal");
			
					if (lineType == 2 && mode == 1) {
						// finally, user name
						tmpString = getItem(buf,':',10);
						// FIXME yeah, acutally we do, but in the constructor for pkiName make 0-len names convert to i18n("Unknown") and prepare for sorting...
						// we dont load empty names in pub line -- stefan
						if (tmpString.length() > 0) newName = new pkiName(newKey,extractDisplayName(tmpString.data()),extractEmailAddress(tmpString.data()));
						inSubKey = 0;
					}

 					tmpString = getItem(buf,':',9);
 					strncpy(c,tmpString.data(),1);
 					switch (c[0]) {
 						case '-':
 						case 'q':
 						case 'e':
 							newKey->setTrust(0);
 							break;
 						case 'n':
 							newKey->setTrust(1);
 							break;
 						case 'm':
 							newKey->setTrust(2);
 							break;
 						case 'f':
 							newKey->setTrust(3);
 							break;
 						case 'u':
 							newKey->setTrust(4);
 							break;
 					}
 					if (lineType == 1) newKey->setTrust(4);
					break;
				
				case 4:
					if (mode == 1) {
#ifdef MY_DEBUG
printf("\tNew Name!\n");
#endif
						tmpString = getItem(buf,':',10);
						newName = new pkiName(newKey,extractDisplayName(tmpString.data()),extractEmailAddress(tmpString.data()));
						inSubKey = 0;
						winProgress->setLabelText(newName->getDisplayName()->data()); // I want to show the name...
					} // we only load names under the public keyring!
					break;
				case 5:
					if (mode == 1) {
#ifdef MY_DEBUG
printf("\tNew Signature!\n");
#endif
						tmpString.sprintf("0%s",getItem(buf,':',5));
						if (tmpString.length() > 1) {
							if (inSubKey == 0)
								newSig = new pkiSig(newName,tmpString.data());
							else
								newSig = new pkiSig(newSubKey,tmpString.data());
						} // else it's an invalid key...
					} // we only load names/subkeys, therefore sigs, on public keyring!
					break;
				case 7:
					// fingerprint...
#ifdef MY_DEBUG
printf("\tNew Fingerprint!\n");
#endif
					tmpString = getItem(buf,':',10);
					newKey->setFingerprint(tmpString);
					break;
				case 6:
					// subkey...
					if (mode == 1) {
#ifdef MY_DEBUG
printf("\tNew Subkey!\n");
#endif
						// keyID
						tmpString.sprintf("0%s",getItem(buf,':',5));
						newSubKey = new pkiSubKey(newKey,tmpString.data());
	
						// next, key size...
						tmpString = getItem(buf,':',3);
						newSubKey->setKeySize(tmpString.data());
	
						// next, GPG key type...
						tmpString = getItem(buf,':',4);
						if (tmpString == "1") newSubKey->setKeyType("RSA");
						if (tmpString == "16") newSubKey->setKeyType("ElGamal");
						if (tmpString == "17") newSubKey->setKeyType("DSA");
						if (tmpString == "20") newSubKey->setKeyType("ElGamal");
					
						inSubKey = 1;
					}
					break;
				case 8:
					// revokation...
					// I hope this is correct. "rev" is not listed in the documentation -- stefan
					if (mode == 1) {
						tmpString.sprintf("0%s",getItem(buf,':',5));
						if (tmpString == newKey->getKeyID()->data()) newKey->isRevoked = TRUE;
					}
					break;
			}
		}
	}
	fclose(outFile);
}

void myProfile::loadPGP2Keyring(const char *appString, int mode, QProgressDialog *winProgress)
{
	pkiKey *newKey = NULL;
	pkiName *newName = NULL;
	pkiSig *newSig = NULL;
	FILE *inFile, *outFile; // DO NOT FREE ME (I don't think so, that is. )
	char buf[65530], keyType[KEY_TYPE_LENGTH]; // FIXME: Un-hard code these if possible.
	QString tmpString, tmpString2;
	int lineType=0;

	pipe_pgp(appString,&inFile,&outFile);
	qApp->processEvents();

	if (outFile == NULL) return;
	while (!feof(outFile)) {
		incrQPD(winProgress);
		qApp->processEvents();
		fgets(buf,65530,outFile);
#ifdef MY_DEBUG
printf("PGP2 In: %s\n",buf);
#endif

		strncpy(keyType,buf,3);
		qApp->processEvents();
		lineType = 0; // unknown yet...
		if (strncmp(keyType,"sec",3) == 0) lineType = 1; // secret key
		if (strncmp(keyType,"pub",3) == 0) lineType = 2; // public key
		if (strncmp(keyType,"sig",3) == 0) lineType = 5; // signature
		if (strncmp(keyType,"   ",3) == 0) lineType = 4; // name OR fingerprint...

                // from this point on, only check against lineType, not keyType -- Stefan
		
		switch (lineType) {
			case 1:
			case 2:
#ifdef MY_DEBUG
printf("New Key!\n");
#endif
				// eat leading "sec" or "pub"
				strtok(buf, " ");
					
				// next, key size...
				tmpString2 = strtok('\0',"/");
	
				// next, keyID
				tmpString.sprintf("0x%s",strtok('\0'," "));

				// Here, we search for this key. We may already have it, if so, use existing record...
				newKey = getKey(tmpString.data());
				if (newKey == NULL) {
					newKey = new pkiKey(this,tmpString.data());
					tmpString2 = tmpString2.stripWhiteSpace();
					newKey->setKeySize(tmpString2);
					newKey->setKeyType("RSA");
				}

				// creation date...
				tmpString = strtok('\0'," ");
				newKey->setCreateDate(tmpString);
				// expire date (PGP-2.6.3i doesnt support expiring of keys, so...   -- stefan)
					
				if (lineType == 1) newKey->haveSecret = TRUE;

				// finally, user name. Only load the names in the PUBLIC keyring.
				// We only load the secret one to pick up if we have the secret as well as public keys...
				// only load names ONE TIME -- stefan
				if (mode != 2) break;
#ifdef MY_DEBUG
printf("\tNew Name!\n");
#endif

				tmpString = strtok('\0',"\n");
				tmpString2 = tmpString.stripWhiteSpace();
  				if (tmpString2.find("*** KEY REVOKED ***") >= 0) newKey->isRevoked = TRUE;
				// We've got a name!!!
				newName = new pkiName(newKey,extractDisplayName(tmpString2.data()),extractEmailAddress(tmpString2.data()));

				// We have to use sscanf with a space in the first position
				// of the format string to ignore the leading spaces before the name
				// sscanf(buf," %[^\n]",tmpString); // Old version I want to keep until I trust stripWhiteSpace()
				winProgress->setLabelText(newName->getDisplayName()->data()); // I want to show the name...
				break;
			case 4:
				// Only load the names in the PUBLIC keyring to avoid duplicates.
				// Only load fingerprints in the PUBLIC keyring.
				if (mode == 2) {
#ifdef MY_DEBUG
printf("\tNew Name!\n");
#endif
					// We've got a name!!!
					tmpString = buf;
					tmpString2 = tmpString.stripWhiteSpace();
					newName = new pkiName(newKey,extractDisplayName(tmpString2.data()),extractEmailAddress(tmpString2.data()));

					// We have to use sscanf with a space in the first position
					// of the format string to ignore the leading spaces before the name
					// sscanf(buf," %[^\n]",tmpString); // Old version I want to keep until I trust stripWhiteSpace()
					winProgress->setLabelText(newName->getDisplayName()->data()); // I want to show the name...
				} else if (mode == 3) {
#ifdef MY_DEBUG
printf("\tNew Fingerprint!\n");
#endif
					// We've got a fingerprint!!!
					tmpString = buf;
					if (tmpString.find("Key fingerprint =")<0) break;
					// This is no fingerprint, but a name
					tmpString2 = tmpString.mid(tmpString.find('=') + 1,tmpString.length() - tmpString.find('=') - 1);
					tmpString = tmpString2.stripWhiteSpace();
					newKey->setFingerprint(tmpString);
				}
				break;
			case 5:
#ifdef MY_DEBUG
printf("\tNew Signature!\n");
#endif
				// eat leading "sig"
				strtok(buf, " ");
					
				tmpString.sprintf("0x%s",strtok('\0'," "));
				newSig = new pkiSig(newName,tmpString.data());
				break;
		}
	}
	fclose(outFile);
}

void myProfile::loadPGP2Validity(const char *appString, QProgressDialog *winProgress)
{
	pkiKey *newKey = NULL;
	FILE *inFile, *outFile; // DO NOT FREE ME (I don't think so, that is. )
	char buf[65530], c[2];
	QString tmpString, KeyId, Validity, Trust;

	pipe_pgp(appString,&inFile,&outFile);
	qApp->processEvents();

	if (outFile == NULL) return;

	winProgress->setLabelText(i18n("Loading Validity..."));
		
	while (!feof(outFile)) {
		incrQPD(winProgress);
		qApp->processEvents();
		fgets(buf,65530,outFile);
		tmpString = buf;
		if (tmpString.find("KeyID    Trust     Validity  User ID") > -1)	break;
	}
	
	while (!feof(outFile)) {
		incrQPD(winProgress);
		qApp->processEvents();
		fgets(buf,65530,outFile);
  		tmpString = buf;
  		if (tmpString.mid(2,1) != " ") {       // Startpoint of Key-ID, empty when name or signature
  			KeyId = tmpString.mid(2,8);
  			KeyId = "0x" + KeyId;
  			Validity = tmpString.mid(21,9);
  			Trust = tmpString.mid(11,9);
  		
#ifdef MY_DEBUG
printf("KeyId %s  ",KeyId.data());
printf("Validity %s\n",Validity.data());
#endif

			newKey = getKey(KeyId.data());
			if (newKey == NULL) continue;              
			// Houston, we have a problem ....
				
			strncpy(c,Validity.data(),1);
			switch (c[0]) {
				case 'u':
					newKey->setValidity(0);
					break;
				case 'n':
					newKey->setValidity(1);
					break;
				case 'm':
					newKey->setValidity(2);
					break;
				case 'c':
					newKey->setValidity(3);
					break;
			}
			
			if (Trust.contains("untrusted"))
				newKey->setTrust(1);
			if (Trust.contains("undefined"))
				newKey->setTrust(0);
			if (Trust.contains("marginal"))
				newKey->setTrust(2);
			if (Trust.contains("complete"))
				newKey->setTrust(3);
			
			if (newKey->haveSecret) {
				newKey->setValidity(4);
				newKey->setTrust(4);
			}
  		}

	}
	fclose(outFile);
}


void myProfile::loadPGP5Keyring(const char *appString, QProgressDialog *winProgress)
{
	pkiKey *newKey = NULL;
	pkiName *newName = NULL;
	pkiSig *newSig = NULL;
	FILE *inFile, *outFile; // DO NOT FREE ME (I don't think so, that is. )
	char buf[65530], keyType[KEY_TYPE_LENGTH]; // FIXME: Un hard-code these if possible.
	QString tmpString, tmpString2;
	int lineType=0;
	time_t localdate;
	tm *gmtdate;
	char date[12];

	winProgress->setLabelText(i18n("Loading keyring..."));
	pipe_pgp(appString,&inFile,&outFile);
	if (outFile == NULL) return;
	while (!feof(outFile)) {
		incrQPD(winProgress);
		qApp->processEvents();
		fgets(buf,65530,outFile);
#ifdef MY_DEBUG
printf("PGP5 In: %s\n",buf);
#endif

		if (strlen(buf) >= 5) {
			strncpy(keyType,strtok(buf," "),KEY_TYPE_LENGTH);
			lineType=0; // unknown default...
			if (strcmp(keyType,"sec") == 0)  lineType = 1; // secret line...
			if (strcmp(keyType,"sec+") == 0) lineType = 1; // secret line...
			if (strcmp(keyType,"pub") == 0)  lineType = 2; // public key
			if (strcmp(keyType,"pub@") == 0) lineType = 2; // disabled public key
			if (strcmp(keyType,"sub") == 0)  lineType = 3; // subkey...
			if (strcmp(keyType,"uid") == 0)  lineType = 4; // userid..
			if (strcmp(keyType,"f16") == 0)  lineType = 7; // fingerprint...
			if (strcmp(keyType,"f20") == 0)  lineType = 7; // fingerprint...
			if (strcmp(keyType,"sig") == 0)  lineType = 5; // signature...
			if (strcmp(keyType,"SIG") == 0)  lineType = 5; // signature...
			if (strcmp(keyType,"sig?") == 0) lineType = 5; // signature...
	
			qApp->processEvents();
			switch (lineType) {
				case 1:
				case 2:
#ifdef MY_DEBUG
printf("New Key!\n");
#endif
					// next, key size...
					tmpString = strtok('\0'," ");
					tmpString2 = strtok('\0'," ");
					newKey = new pkiKey(this,tmpString2.data());

					newKey->setKeySize(tmpString);
					newKey->haveSecret = (lineType == 1);
					if (strcmp(keyType,"pub@") == 0) newKey->isDisabled = TRUE;
					
					// next, 2 dates
					tmpString = strtok('\0'," ");
					newKey->setCreateDate(tmpString);
					tmpString = strtok('\0'," ");
					newKey->setExpireDate(tmpString);
					
					if (strcmp("*REVOKED*",newKey->getExpireDate()->data()) == 0) {
						newKey->isRevoked = TRUE;
						// I have to insert a dummy name to get same layout as in the other PGP programs  -- stefan
						newName = new pkiName(newKey,"*REVOKED*","*REVOKED*");
					}
					
					time (&localdate);
					gmtdate = gmtime (&localdate);
					strftime (date,sizeof date,"%Y-%m-%d",gmtdate);
					if (strcmp("----------",newKey->getExpireDate()->data()) != 0 && strcmp("*REVOKED*",newKey->getExpireDate()->data()) != 0 && strncmp(date,newKey->getExpireDate()->data(),12) > 0)
						newKey->isExpired = TRUE;
					
					// next, key type...
					newKey->setKeyType(strtok('\0'," "));
					if (strcasecmp(newKey->getKeyType()->data(),"DSS") == 0) newKey->setKeyType("DH/DSS");
					break;
				case 4:
#ifdef MY_DEBUG
printf("\tNew Name!\n");
#endif
					tmpString = strtok('\0',"\n");
					tmpString2 = tmpString.stripWhiteSpace();
					newName = new pkiName(newKey,extractDisplayName(tmpString2.data()),extractEmailAddress(tmpString2.data()));

					winProgress->setLabelText(newName->getDisplayName()->data()); // I want to show the name...
					break;
				case 5:
#ifdef MY_DEBUG
printf("\tNew Signature!\n");
#endif
					// Signature for PGP 5
					// sig	0x???????? date name ... name <email address>
					newSig = new pkiSig(newName,strtok('\0'," "));
					break;
			case 7:
#ifdef MY_DEBUG
printf("\tNew Fingerprint!\n");
#endif
					// Fingerprint
					tmpString = strtok('\0',"\n");
					tmpString2 = tmpString.mid(tmpString.find('=') + 1,tmpString.length() - tmpString.find('=') - 1);
					tmpString = tmpString2.stripWhiteSpace();
					newKey->setFingerprint(tmpString);
					break;
			}
		}
	}
	fclose(outFile);
	return;
}


void myProfile::loadPGP6Keyring(const char *appString, int mode, QProgressDialog *winProgress)
{
 	pkiKey *newKey = NULL;
 	pkiSubKey *newSubKey = NULL;
 	pkiName *newName = NULL;
 	pkiSig *newSig = NULL;
 	FILE *inFile, *outFile; // DO NOT FREE ME (I don't think so, that is. )
 	char buf[65530], keyType[KEY_TYPE_LENGTH]; // FIXME: Un-hard code these if possible.
 	QString tmpString, tmpString2;
 	int lineType=0, inSubKey=0;

 	pipe_pgp(appString,&inFile,&outFile);
 	qApp->processEvents();

 	if (outFile == NULL) return;
 	while (!feof(outFile)) {
		incrQPD(winProgress);
 		qApp->processEvents();
 		fgets(buf,65530,outFile);
#ifdef MY_DEBUG
printf("PGP6 In: %s\n",buf);
#endif

 		strncpy(keyType,buf,4);
 		qApp->processEvents();
 		lineType = 0; // unknown yet...
 		if (strncmp(keyType,"RSA",3) == 0) lineType = 1; // RSA key
 		if (strncmp(keyType,"DSS",3) == 0) lineType = 2; // DSA key
 		if (strncmp(keyType," DH",3) == 0) lineType = 3; // subkey
 		if (strncmp(keyType,"sig",3) == 0) lineType = 5; // signature
 		if (strncmp(keyType,"   ",3) == 0) lineType = 4; // name OR fingerprint...

                // from this point on, only check against lineType, not keyType -- Stefan
 		
 		switch (lineType) {
 			case 1:
 			case 2:
#ifdef MY_DEBUG
printf("New Key!\n");
#endif
 				// eat leading keyType
 				strtok(buf, " ");
 					
 				// next, key size...
 				tmpString2 = strtok('\0'," ");
 	
 				// next, keyID
 				tmpString = strtok('\0'," ");

 				// Here, we search for this key. We may already have it, if so, use existing record...
 				newKey = getKey(tmpString.data());
 				if (newKey == NULL) {
 					newKey = new pkiKey(this,tmpString.data());
 					newKey->setKeySize(tmpString2.data());
 					if (lineType == 1)
 						newKey->setKeyType("RSA");        // we have to terminate the string manually --stefan
 					if (lineType == 2)
 						newKey->setKeyType("DSA");        // we have to terminate the string manually --stefan
 					
 					if (strncmp(keyType,"DSS@",4) == 0 || strncmp(keyType,"RSA@",4) == 0)
 						newKey->isDisabled = TRUE;
 				}

				// creation date...
				tmpString = strtok('\0'," ");
				newKey->setCreateDate(tmpString);
 					
 				if (mode == 1) newKey->haveSecret = TRUE;
 				inSubKey=0;

 				// finally, user name. Only load the names in the PUBLIC keyring.
 				// We only load the secret one to pick up if we have the secret as well as public keys...
 				// only load names ONE TIME -- stefan
 				if (mode != 2) break;
 				
 				tmpString = strtok('\0',"\n");
 				tmpString2 = tmpString.stripWhiteSpace();

 				if (tmpString2.isEmpty())
 					break;      // the DSS line has no further infos   :-(
 				
 				if (tmpString2.find("expires") >= 0) {
					tmpString2 = tmpString2.mid(tmpString2.find(' ') + 1,tmpString2.length() - tmpString2.find(' ') - 1);
 					newKey->setExpireDate(tmpString2); // we only found infos about Key Expiring
 					break;
 				}
 				if (tmpString2.find("*** KEY EXPIRED ***") >= 0) {
 					newKey->setExpireDate(tmpString2); // we only found infos about Key Expiring
					newKey->isExpired = TRUE;
 					break;
 				}
 				
 				if (tmpString2.find("*** KEY REVOKED ***") >= 0) {
					newKey->isRevoked = TRUE;
 					break;
 				}
#ifdef MY_DEBUG
printf("\tNew Name!\n");
#endif
 				// We've got a name!!!
 				newName = new pkiName(newKey,extractDisplayName(tmpString2.data()),extractEmailAddress(tmpString2.data()));

 				// We have to use sscanf with a space in the first position
 				// of the format string to ignore the leading spaces before the name
 				// sscanf(buf," %[^\n]",tmpString); // Old version I want to keep until I trust stripWhiteSpace()

 				winProgress->setLabelText(newName->getDisplayName()->data()); // I want to show the name...
 				break;
 			case 3:
 				// Only load the subkeys in the PUBLIC keyring to avoid duplicates.
 				if (mode != 2) break;
 				
#ifdef MY_DEBUG
printf("\tNew Subkey!\n");
#endif
 				// eat leading keyType
 				strtok(buf, " ");
 					
 				// next, key size...
				tmpString = strtok('\0'," ");
 	
 				// next, keyID
 				newSubKey = new pkiSubKey(newKey,strtok('\0'," "));
 				newSubKey->setKeyType("DSA");
 				newSubKey->setKeySize(tmpString);
 				
				// creation date...
				tmpString = strtok('\0'," ");
				newSubKey->setCreateDate(tmpString);
 	
 				inSubKey = 1;

 				// last DH subkey carry the name information for DSS key!!!
 				tmpString = strtok('\0',"\n");
 				tmpString2 = tmpString.stripWhiteSpace();
 				if (tmpString2.isEmpty())
 					break;    // this DH line has no further infos   :-(

 				if (tmpString2.find("expires") >= 0) {
					tmpString2 = tmpString2.mid(tmpString2.find(' ') + 1,tmpString2.length() - tmpString2.find(' ') - 1);
 					newSubKey->setExpireDate(tmpString2); // we only found infos about Key Expiring
 					break;
 				}
 				if (tmpString2.find("*** KEY EXPIRED ***") >= 0) {
 					newSubKey->setExpireDate(tmpString2); // we only found infos about Key Expiring
 					break;
 				}

#ifdef MY_DEBUG
printf("\tNew Name!\n");
#endif
 				newName = new pkiName(newKey,extractDisplayName(tmpString2.data()),extractEmailAddress(tmpString2.data()));

 				// We have to use sscanf with a space in the first position
 				// of the format string to ignore the leading spaces before the name
 				// sscanf(buf," %[^\n]",tmpString); // Old version I want to keep until I trust stripWhiteSpace()

 				winProgress->setLabelText(newName->getDisplayName()->data()); // I want to show the name...
 				
 				break;
 			case 4:
 				// Only load the names in the PUBLIC keyring to avoid duplicates.
 				// Only load fingerprints in the PUBLIC keyring.
 				if (mode == 2) {
#ifdef MY_DEBUG
printf("\tNew Name!\n");
#endif

 					tmpString = buf;
 					tmpString2 = tmpString.stripWhiteSpace();

 					// We've got a name!!!
	 				newName = new pkiName(newKey,extractDisplayName(tmpString2.data()),extractEmailAddress(tmpString2.data()));

 					// We have to use sscanf with a space in the first position
 					// of the format string to ignore the leading spaces before the name
 					// sscanf(buf," %[^\n]",tmpString); // Old version I want to keep until I trust stripWhiteSpace()

 					winProgress->setLabelText(newName->getDisplayName()->data()); // I want to show the name...
 				} else if (mode == 3) {
#ifdef MY_DEBUG
printf("\tNew Fingerprint!\n");
#endif
 					// We've got a fingerprint!!!
 					tmpString = buf;
 					if (tmpString.find("Key fingerprint =")<0)      // This is no fingerprint, but a name
 						break;			
					tmpString2 = tmpString.mid(tmpString.find('=') + 1,tmpString.length() - tmpString.find('=') + 1);
 					tmpString = tmpString2.stripWhiteSpace();
 					newKey->setFingerprint(tmpString);
 				}
 				break;
 			case 5:
 				if (mode != 2) break;
#ifdef MY_DEBUG
printf("\tNew Signature!\n");
#endif

 				// eat leading "sig"
 				strtok(buf, " ");
  				newSig = new pkiSig(newName,strtok('\0'," "));
 				break;
 		}
 	}
 	fclose(outFile);
}

void myProfile::loadPGP65Validity(const char *appString, QProgressDialog *winProgress)
{
 	pkiKey *newKey = NULL;
 	FILE *inFile, *outFile; // DO NOT FREE ME (I don't think so, that is. )
 	char buf[65530], c[2];
 	QString tmpString, KeyId, Validity, Trust;

 	pipe_pgp(appString,&inFile,&outFile);
 	qApp->processEvents();

 	if (outFile == NULL) return;

 	winProgress->setLabelText(i18n("Loading Validity..."));
	winProgress->show();
	qApp->processEvents();
 		
 	while (!feof(outFile)) {
		incrQPD(winProgress);
	 	winProgress->setLabelText(i18n("Loading Validity..."));
		winProgress->show();
 		qApp->processEvents();
 		fgets(buf,65530,outFile);
 		tmpString = buf;
 		if (tmpString.find("KeyID      Trust     Validity  User ID") > -1)	break;
 	}
 	
 	while (!feof(outFile)) {
		incrQPD(winProgress);
		qApp->processEvents();
 		fgets(buf,65530,outFile);
   		tmpString = buf;
   		if (tmpString.mid(2,1) == "0") {       // Startpoint of Key-ID, empty when name or signature
   			KeyId = tmpString.mid(2,10);
   			Validity = tmpString.mid(23,9);
   			Trust = tmpString.mid(13,9);
			winProgress->setLabelText(KeyId.data());
   		
 			newKey = getKey(KeyId.data());
 			if (newKey == NULL) continue;
 			// Houston, we have a problem ....
 				
#ifdef MY_DEBUG
printf("KeyId %s  ",KeyId.data());
printf("Validity %s\n",Validity.data());
#endif

 				
 			strncpy(c,Validity.data(),1);
 			switch (c[0]) {
 				case 'u':
 					newKey->setValidity(0);
 					break;
 				case 'i':
 					newKey->setValidity(1);
 					break;
 				case 'm':
 					newKey->setValidity(2);
 					break;
 				case 'c':
 					newKey->setValidity(3);
 					break;
 			}
 			
 			if (Trust.contains("untrusted"))
 				newKey->setTrust(1);
 			if (Trust.contains("undefined"))
 				newKey->setTrust(0);
 			if (Trust.contains("marginal"))
 				newKey->setTrust(2);
 			if (Trust.contains("complete"))
 				newKey->setTrust(3);
 			
 			if (newKey->haveSecret == TRUE && newKey->getValidity() > 1) {
 				newKey->setValidity(4);
 				newKey->setTrust(4);
 			}
   		}

 	}
 	fclose(outFile);
}


pkiKey *myProfile::getKey(const char *keyID)
{
	// FIXME: Chris, use a QStrList and make an index of KeyIDS!!
	// FIXME: setup in the constructors of myProfile and pkiKey
	unsigned int l=0;
	pkiKey *pk = NULL;
	for (l=0;l<keyring.count();l++) {
		pk = keyring.at(l); // for debug purposes // FIXME
		if (strcasecmp(pk->getKeyID()->data(),keyID) == 0) return keyring.at(l);
	}
	return NULL;
}

pkiSubKey *myProfile::getSubKey(const char *subkeyID, QList<pkiSubKey> *subkeys)
{
	unsigned int l=0;
	if (subkeys == NULL) return NULL;
	for (l=0;l<subkeys->count();l++) {
		if (strcmp(subkeys->at(l)->getKeyID()->data(),subkeyID) == 0) return subkeys->at(l);
	}
	return NULL;
}

pkiSig *myProfile::getSig(const char *keyID, QList<pkiSig> *sigs)
{
	unsigned int l=0;
	if (sigs == NULL) return NULL;
	for (l=0;l<sigs->count();l++) {
		if (strcmp(sigs->at(l)->getSigningKeyID()->data(),keyID) == 0) return sigs->at(l);
	}
	return NULL;
}

int myProfile::importKey_File(const char *fileName)
{
	int retVal = -1;
	QString appString;

	if (fileName == NULL) {
		KMessageBox::sorry(NULL,i18n("You must enter or select a file!"));
		return -1;
	}

	switch(pgpVersion) {
		case 6:
			appString.sprintf("%s -ka %s %s",pgpExecs.at(0), fileName, keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 5:
			appString.sprintf("%s -a %s %s",pgpExecs.at(0), fileName, keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 2:
			appString.sprintf("%s -ka %s",pgpExecs.at(0), fileName);
			retVal = runProcessFG(appString.data());
			break;
		case 1:
			appString.sprintf("%s %s --import %s",pgpExecs.at(0),keyringsToUse().data(),fileName);
			retVal = runProcessFG(appString.data());
			break;
		default:
			showBadProfileDialog();
			break;
	}
	loadKeyring(); // have to refresh the list...
	return retVal;
}

int myProfile::importKey_Clipboard()
{
	QString *tmpS;
	myClipboard *tmpClip = new myClipboard();
	int retVal = 0;

	tmpS = getSafeTmpNameQ();

	tmpClip->saveFromClipboard(tmpS->data());
	retVal = importKey_File(tmpS->data());
	unlink(tmpS->data());

	delete tmpS;
	return retVal;
}

int myProfile::importKey_Server(QStrList *keyList, bool forceFullUpdate) {
	unsigned int i = 0;
	int j = 0, k = 0;

	for (i=0;i<keyList->count();i++) {
		k = importKey_Server(keyList->at(i),FALSE);
		if (k != 0) j = -1;
	}
	if (forceFullUpdate) loadKeyring();
	return k;
}

int myProfile::importKey_Server(const char *keyID, bool forceFullUpdate)
{
	int retVal = -1;
	QString appString;
	QString *tmpKey;
	KConfig *cfg = NULL;
	QString *tmpOptions = new QString("");

	if (keyID == NULL) {
		KMessageBox::sorry(NULL,i18n("You must enter or select a key!"));
		return -1;
	}

	cfg = kapp->config();
	if (cfg) {
		cfg->setGroup(profileName);
		if (cfg->readEntry("HonorHTTPProxy","0").compare("1") == 0) tmpOptions->append(" --honor-http-proxy ");
	}

	switch(pgpVersion) {
		case 6:
			tmpKey = getSafeTmpNameQ();
			appString.sprintf("%s -kx %s %s ldap://%s",pgpExecs.at(0),keyID,tmpKey->data(),keyServer.data());
			retVal = runProcessFG(appString.data(),1);
#ifdef MY_DEBUG
printf("retVal = %d\n",retVal);
#endif
			appString.sprintf("%s -ka %s",pgpExecs.at(0),tmpKey->data());
			retVal = runProcessFG(appString.data(),1);
			appString.sprintf("rm -f %s",tmpKey->data());
			retVal = runProcessFG(appString.data(),0);
			if (tmpKey) delete tmpKey;
			break;
		case 5:
			appString.sprintf("%s -a %s %s",pgpExecs.at(0),keyID,keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 2:
			KMessageBox::sorry(NULL,i18n("Sorry, but your encryption program does not support that feature."));
			break;
		case 1:
			appString.sprintf("%s %s %s --keyserver %s --recv-keys %s",pgpExecs.at(0),keyringsToUse().data(),tmpOptions->data(),keyServer.data(),keyID);
			retVal = runProcessFG(appString.data());
			break;
		default:
			showBadProfileDialog();
			break;
	}
	if (forceFullUpdate) loadKeyring();
	delete tmpOptions;
	return retVal;
}

int myProfile::editKey(const char *keyID)
{
	int retVal = -1;
	QString appString;

	if (keyID == NULL) {
		KMessageBox::sorry(NULL,i18n("You must enter or select a key!"));
		return -1;
	}
	if (pgpVersion == 0) {
		showBadProfileDialog();
		return -1;
	}

	switch(pgpVersion) {
		case 6:
			appString.sprintf("%s -ke %s %s",pgpExecs.at(0),keyID,keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 5:
			appString.sprintf("%s -e %s %s",pgpExecs.at(0),keyID,keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 2:
			appString.sprintf("%s -ke %s",pgpExecs.at(0),keyID);
			retVal = runProcessFG(appString.data());
			break;
		case 1:
			appString.sprintf("%s %s --edit-key %s",pgpExecs.at(0),keyringsToUse().data(),keyID);
			retVal = runProcessFG(appString.data());
			break;
		default:
			showBadProfileDialog();
			break;
	}
	loadKeyring();
	return retVal;
}

int myProfile::disEnableKey(const char *keyID)
{
	int retVal = -1;
	QString appString;

	if (keyID == NULL) {
		KMessageBox::sorry(NULL,i18n("You must enter or select a key!"));
		return -1;
	}
	if (pgpVersion == 0) {
		showBadProfileDialog();
		return -1;
	}

	switch(pgpVersion) {
		case 6:
			appString.sprintf("%s -kd %s",pgpExecs.at(0),keyID);
			retVal = runProcessFG(appString.data());
			break;
		case 5:
			appString.sprintf("%s -d %s %s",pgpExecs.at(0),keyID,keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 2:
			showUnsupportedDialog();
			break;
		case 1:
			appString.sprintf("%s %s --edit-key %s",pgpExecs.at(0),keyringsToUse().data(),keyID);
			retVal = runProcessFG(appString.data());
			break;
		default:
			showBadProfileDialog();
			break;
	}
	loadKeyring();
	return retVal;
}

int myProfile::removeKey(const char *keyID)
{
	// PRIVATE FUNCTION!!
	QString appString;
	pkiKey *tmpKey = NULL;
	int retVal = -1;

	if (keyID == NULL) {
		showBadKeyIDDialog();
		return -1;
	}
	if (pgpVersion == 0) {
		showBadProfileDialog();
		return -1;
	}

	switch(pgpVersion) {
		case 6:
			appString.sprintf("%s -kr %s %s",pgpExecs.at(0),keyID,keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 5:
			appString.sprintf("%s -r %s %s",pgpExecs.at(0),keyID,keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 2:
			appString.sprintf("%s -kr %s",pgpExecs.at(0),keyID);
			retVal = runProcessFG(appString.data());
			break;
		case 1:
			tmpKey = getKey(keyID);
			if (tmpKey) if (tmpKey->haveSecret) {
				appString.sprintf("%s %s --delete-secret-key %s",pgpExecs.at(0),keyringsToUse().data(),keyID);	
				retVal = runProcessFG(appString.data());
			}
			appString.sprintf("%s %s --delete-key %s",pgpExecs.at(0),keyringsToUse().data(),keyID);
			retVal = runProcessFG(appString.data());
			break;
		default:
			showBadProfileDialog();
			break;
	}
	loadKeyring();
	return retVal;
}

int myProfile::revokeKey(const char *keyID)
{
	// PRIVATE FUNCTION!!
	QString appString, txtRevokCert;
	int retVal = -1;
	int tmp = 0;

	if (keyID == NULL) {
		KMessageBox::sorry(NULL,i18n("You must enter or select a key!"));
		return -1;
	}
	if (pgpVersion == 0) {
		showBadProfileDialog();
		return -1;
	}

	// Note: warningContinueCancel doesn't have defaults like the others in QT 2.2.2 - look at the header file. ;( cdw
	tmp = KMessageBox::warningContinueCancel(NULL,i18n("Are you REALLY REALLY sure you want to revoke this key!! This may not be undoable!!"),i18n("Warning"),i18n("Continue"));
	if (tmp != KMessageBox::Continue) return -1;

	switch(pgpVersion) {
		case 6:
			appString.sprintf("%s -kd %s",pgpExecs.at(0),keyID);
			retVal = runProcessFG(appString.data());
			break;
		case 5:
			appString.sprintf("%s --revoke %s %s",pgpExecs.at(0),keyID,keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 2:
			appString.sprintf("%s -kd %s",pgpExecs.at(0),keyID);
			retVal = runProcessFG(appString.data());
			break;
		case 1:
			KMessageBox::information(NULL,i18n("When GnuPG revokes a key, it saves it as a Revocation Certificate.\nTo actually revoke the key, you must import this certificate,\nthen send it to the key servers and give it everyone who uses your key.\nI will launch GnuPG to revoke the key.\nAfter, YOU MUST IMPORT THAT REVOCATION CERTIFICATE TO ACTUALLY REVOKE YOUR KEY."),NULL,"NotifyGnuPG10RevokCert");
			txtRevokCert = KFileDialog::getSaveFileName(0,0,NULL,0);
			if (txtRevokCert.isNull()) return -1;
			appString.sprintf("%s %s -o %s --gen-revoke %s",pgpExecs.at(0),keyringsToUse().data(),txtRevokCert.data(),keyID);
			retVal = runProcessFG(appString.data());
			break;
		default:
			showBadProfileDialog();
			break;
	}
	loadKeyring();
	return retVal;
}

int myProfile::revokeSignature(const char *keyID=NULL)
{
	QString appString;
	int retVal = -1;

	if (keyID == NULL) {
		KMessageBox::sorry(NULL,i18n("You must enter or select a key!"));
		return -1;
	}
	if (pgpVersion == 0) {
		showBadProfileDialog();
		return -1;
	}

	switch(pgpVersion) {
		case 6:
			appString.sprintf("%s -krs %s %s",pgpExecs.at(0),keyID,keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 5:
			appString.sprintf("%s --revokes %s %s",pgpExecs.at(0),keyID,keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 2:
			appString.sprintf("%s -krs %s",pgpExecs.at(0),keyID);
			retVal = runProcessFG(appString.data());
			break;
		case 1:
			appString.sprintf("%s %s --edit-key %s",pgpExecs.at(0),keyringsToUse().data(),keyID);
			retVal = runProcessFG(appString.data());
			break;
		default:
			showBadProfileDialog();
			break;
	}
	loadKeyring();
	return retVal;
}

int myProfile::addName(const char *keyID=NULL)
{
	QString appString;
	int retVal = -1;

	if (keyID == NULL) {
		KMessageBox::sorry(NULL,i18n("You must enter or select a key!"));
		return -1;
	}
	if (pgpVersion == 0) {
		showBadProfileDialog();
		return -1;
	}

	switch(pgpVersion) {
		case 6:
			appString.sprintf("%s -ke %s %s",pgpExecs.at(0),keyID,keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 5:
			appString.sprintf("%s -e %s %s",pgpExecs.at(0),keyID,keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 2:
			appString.sprintf("%s -ke %s",pgpExecs.at(0),keyID);
			retVal = runProcessFG(appString.data());
			break;
		case 1:
			appString.sprintf("%s %s --edit-key %s",pgpExecs.at(0),keyringsToUse().data(),keyID);
			retVal = runProcessFG(appString.data());
			break;
		default:
			showBadProfileDialog();
			break;
	}
	loadKeyring();
	return retVal;
}

int myProfile::genKeyPair()
{
	QString appString;
	int retVal = -1;

	if (pgpVersion == 0) {
		showBadProfileDialog();
		return -1;
	}

	switch(pgpVersion) {
		case 6:
			appString.sprintf("%s -kg",pgpExecs.at(0));
			retVal = runProcessFG(appString.data());
			break;
		case 5:
			appString.sprintf("%s -g %s",pgpExecs.at(0),keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 2:
			appString.sprintf("%s -kg",pgpExecs.at(0));
			retVal = runProcessFG(appString.data());
			break;
		case 1:
			appString.sprintf("%s %s --gen-key",pgpExecs.at(0),keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		default:
			showBadProfileDialog();
			break;
	}
	loadKeyring();
	return retVal;
}

int myProfile::exportKey_Server(const char *keyID)
{
	QString appString;
	int retVal = -1;

	if (keyID == NULL) {
		KMessageBox::sorry(NULL,i18n("You must enter or select a key!"));
		return -1;
	}
	if (pgpVersion == 0) {
		showBadProfileDialog();
		return -1;
	}

	switch(pgpVersion) {
		case 6:
// FIX ME		appString.sprintf("%s -ke %s %s",pgpExecs.at(0),keyID,keyringsToUse().data());
//			retVal = runProcessFG(appString.data());
			showUnsupportedDialog();
			break;
		case 5:
			showUnsupportedDialog();
			break;
		case 2:
			showUnsupportedDialog();
			break;
		case 1:
			appString.sprintf("%s %s --keyserver %s --send-keys %s",pgpExecs.at(0),keyringsToUse().data(),keyServer.data(),keyID);
			retVal = runProcessFG(appString.data());
			break;
		default:
			showBadProfileDialog();
			break;
	}
	return retVal;
}

int myProfile::exportKey_Clipboard(const char *keyID)
{
	QString *tmpS;
	myClipboard *tmpClip = new myClipboard();
	int retVal = 0;

	tmpS = getSafeTmpNameQ();
	retVal = exportKey_File(keyID,tmpS->data());
	tmpClip->saveIntoClipboard(tmpS->data());
	unlink(tmpS->data());
	delete tmpS;
	return retVal;
}

int myProfile::exportKey_File(const char *keyID,const char *fileName)
{
	int retVal=-1;
	QString appString;

	if (keyID == NULL) {
		KMessageBox::sorry(NULL,i18n("You must enter or select a key!"));
		return -1;
	}

	if (pgpVersion == 0) {
		showBadProfileDialog();
		return -1;
	}

	switch(pgpVersion) {
		case 6:
			appString.sprintf("%s -kxa %s %s %s",pgpExecs.at(0),keyID,fileName,keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 5:
			appString.sprintf("%s -xa %s -o %s %s",pgpExecs.at(0),keyID,fileName,keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 2:
			// Is there *any* way short of piping that we can work around this? -- chris
			KMessageBox::sorry(NULL,i18n("PGP 2 does not allow you to specify an outgoing file to save the exported key to.\nPGP 2 will save the file to the filename you give it,\nwith an \".asc\" appended if the name you give it does not end in .asc already."));
			appString.sprintf("%s -kxa %s %s",pgpExecs.at(0),keyID,fileName);
			retVal = runProcessFG(appString.data());
			break;
		case 1:
			appString.sprintf("%s %s -a -o %s --export %s",pgpExecs.at(0),keyringsToUse().data(),fileName,keyID);
			retVal = runProcessFG(appString.data());
			break;
		default:
			showBadProfileDialog();
			break;
	}
	return retVal;
}

int myProfile::signKey(const char *keyIDToSign, const char *signingKeyID)
{
	int retVal = -1;
	QString appString;
	if (keyIDToSign == NULL || signingKeyID == NULL) {
		KMessageBox::sorry(NULL,i18n("You must enter or select a key!"));
		return -1;
	}
	if (pgpVersion == 0) {
		showBadProfileDialog();
		return -1;
	}

	switch(pgpVersion) {
		case 6:
			appString.sprintf("%s -ks %s -u %s",pgpExecs.at(0),keyIDToSign,signingKeyID);
			retVal = runProcessFG(appString.data());
			break;
		case 5:
			appString.sprintf("%s -s %s -u %s %s",pgpExecs.at(0),keyIDToSign,signingKeyID,keyringsToUse().data());
			retVal = runProcessFG(appString.data());
			break;
		case 2:
			appString.sprintf("%s -ks %s -u %s",pgpExecs.at(0),keyIDToSign,signingKeyID);
			retVal = runProcessFG(appString.data());
			break;
		case 1:
			appString.sprintf("%s %s -u %s --sign-key %s",pgpExecs.at(0),keyringsToUse().data(),signingKeyID,keyIDToSign);
			retVal = runProcessFG(appString.data());
			break;
		default:
			showBadProfileDialog();
			break;
	}
	
	loadKeyring();
	return retVal;
}

int myProfile::removeRevokeKey(const char *keyID=NULL)
{
	// This function is meant to remove if the key selected isn't secret
	int retVal=-1;
	pkiKey *tmpKey = NULL;

	if (keyID == NULL) {
		KMessageBox::sorry(NULL,i18n("You must enter or select a key!"));
		return -1;
	}
	tmpKey = getKey(keyID);

	if (tmpKey) {
		if (tmpKey->haveSecret) retVal = revokeKey(keyID);
		else retVal = removeKey(keyID);
	} else retVal = removeKey(keyID);

	// DO NOT "delete" KEY! It's part of a keyring!!!
	loadKeyring();
	return retVal;
}

int myProfile::decryptFile(const char *file1, const char *file2, bool detachedSig, bool justVerify, QStrList *optionList)
{
	// Private function
	QString pgpAppString;
	QString *options = NULL;
	int retVal = -1;

	if (pgpVersion == 0) {
		showBadProfileDialog();
		return false;
	}

	options = convertQStrListToQString(optionList);

	if (!justVerify) {
		// we're decrypting...
		switch (pgpVersion) {
		case 6:
			pgpAppString.sprintf("%s %s %s -o %s", pgpExecs.at(0), options->data(), file1, file2);
			break;
		case 5:
			pgpAppString.sprintf("%s %s -o %s %s %s", pgpExecs.at(3), options->data(), file2, keyringsToUse().data(), file1 );
			break;
		case 2:
			pgpAppString.sprintf("%s %s %s -o %s", pgpExecs.at(0), options->data(), file1, file2);
			break;
		case 1:
			pgpAppString.sprintf("%s -v %s %s --output %s --decrypt %s", pgpExecs.at(0), options->data(), keyringsToUse().data(), file2, file1);
			break;
		default:
			showBadProfileDialog();
			return false;
		}
	} else {
		// just verifying,
		if (detachedSig) {
			switch (pgpVersion) {
			case 6:
				pgpAppString.sprintf("%s %s +batchmode %s %s -o /dev/null", pgpExecs.at(0), options->data(), file2, file1);
				break;
			case 5:
				// Sadly, PGP 5 is stupid and can't handle both the sig and real file at once... :(
				pgpAppString.sprintf("%s %s %s %s", pgpExecs.at(3), options->data(), file2, keyringsToUse().data() );
				break;
			case 2:
				pgpAppString.sprintf("%s %s %s %s -o /dev/null", pgpExecs.at(0), options->data(), file2, file1);
				break;
			case 1:
				pgpAppString.sprintf("%s -v %s %s --verify %s %s", pgpExecs.at(0), options->data(), keyringsToUse().data(), file2, file1);
				break;
			default:
				showBadProfileDialog();
				return false;
			}
		} else {
			switch (pgpVersion) {
			case 6:
				pgpAppString.sprintf("%s %s +batchmode %s -o /dev/null", pgpExecs.at(0), options->data(), file1);
				break;
			case 5:
				pgpAppString.sprintf("%s %s %s %s -o /dev/null", pgpExecs.at(3), options->data(), file1, keyringsToUse().data() );
				break;
			case 2:
				pgpAppString.sprintf("%s %s %s -o /dev/null", pgpExecs.at(0), options->data(), file1);
				break;
			case 1:
				pgpAppString.sprintf("%s -v %s %s --verify %s", pgpExecs.at(0), options->data(), keyringsToUse().data(), file1);
				break;
			default:
				showBadProfileDialog();
				return false;
			}
		}
	}

	retVal = runProcessFG(pgpAppString,1);  // we want the shell to stay open
	return retVal;
}

int myProfile::encryptFile(const char *inFile, const char *outFile, const char *signingID, const char *recipList,
	bool detachSig, bool signFile, bool encryptFile, bool asciiArmor, bool uniText, bool clearSign, QStrList *optionList)
{
	QString pgpAppString, comment, tmpString;
	QString *options = NULL;
	int retVal = -1;
	KConfig *cfg = NULL;

	cfg = kapp->config();
	cfg->setGroup("__Global");
	comment = cfg->readEntry("Comment", i18n("Made with Geheimnis"));
	options = convertQStrListToQString(optionList);

	switch (pgpVersion) {
	case 6:
		if (comment.isEmpty()) 
			pgpAppString.sprintf("%s %s ",pgpExecs.at(0),options->data());
		else
			pgpAppString.sprintf("%s %s +COMMENT=\\\"%s\\\" ",pgpExecs.at(0),options->data(), comment.data());
		pgpAppString.append(" -"); // we either have to add an "e" and/or "s"...
		if (encryptFile) pgpAppString.append("e");
		if (signFile) {
			pgpAppString.append("s");
			if (detachSig) pgpAppString.append("b");
		}
		if (asciiArmor) pgpAppString.append("a");
		if (uniText) pgpAppString.append("t");
		if (clearSign) pgpAppString.append(" +CLEARSIG=ON ");
		if (signFile) {
			tmpString.sprintf(" -u %s ",signingID);
			pgpAppString.append(tmpString);
		}
		tmpString.sprintf(" -o %s %s %s",outFile,inFile,recipList);
		pgpAppString.append(tmpString);
		retVal = runProcessFG(pgpAppString.data());
		break;
	case 5:
		if (encryptFile)
			pgpAppString.sprintf("%s %s --comment=\\\"%s\\\" ",pgpExecs.at(1),options->data(),comment.data());
		else
			pgpAppString.sprintf("%s %s --comment=\\\"%s\\\" ",pgpExecs.at(2),options->data(),comment.data());

		if (detachSig) pgpAppString.append(" -b ");
		if (encryptFile && signFile) pgpAppString.append(" -s ");
		if (signFile) {
			tmpString.sprintf(" -u %s ",signingID);
			pgpAppString.append(tmpString);
		}
		if (asciiArmor) pgpAppString.append(" -a ");
		if (uniText) pgpAppString.append(" -t ");
		tmpString.sprintf(" %s %s -o %s %s",keyringsToUse().data(),recipList,outFile,inFile);
		pgpAppString.append(tmpString);
		retVal = runProcessFG(pgpAppString.data());
		break;
	case 2:
		pgpAppString.sprintf("%s %s +COMMENT=\\\"%s\\\" ",pgpExecs.at(0),options->data(),comment.data());
		pgpAppString.append(" -"); // we either have to add an "e" and/or "s"...
		if (encryptFile) pgpAppString.append("e");
		if (signFile) {
			pgpAppString.append("s");
			if (detachSig) pgpAppString.append("b");
		}
		if (asciiArmor) pgpAppString.append("a");
		if (uniText) pgpAppString.append("t");
		if (clearSign) pgpAppString.append(" +CLEARSIG=ON ");
		if (signFile) {
			tmpString.sprintf(" -u %s ",signingID);
			pgpAppString.append(tmpString);
		}
		tmpString.sprintf(" -o %s %s %s",outFile,inFile,recipList);
		pgpAppString.append(tmpString);
		retVal = runProcessFG(pgpAppString.data());
		break;
	case 1:
		pgpAppString.sprintf("%s -v --no-batch ",pgpExecs.at(0));
		if (options->length() > 1) pgpAppString.append(options->data());
		tmpString.sprintf("--comment \\\"%s\\\" ",comment.data());
		pgpAppString.append(tmpString.data());
		if (detachSig) pgpAppString.append(" -b ");
		if (encryptFile) pgpAppString.append(" -e ");
		if (signFile) {
			tmpString.sprintf(" -s -u %s ",signingID);
			pgpAppString.append(tmpString);
		}
		if (asciiArmor) pgpAppString.append(" -a ");
		if (clearSign) pgpAppString.append(" --clearsign ");
		if (uniText) pgpAppString.append(" -t ");
		// if we're not encrypting, recipList is just a " ", so we're safe...
		pgpAppString.append(keyringsToUse().data());
		pgpAppString.append(" ");
		pgpAppString.append(recipList);
		pgpAppString.append(" -o ");
		pgpAppString.append(outFile);
		pgpAppString.append(" ");
		pgpAppString.append(inFile);
//		For some unknown reason, this sprintf doesn't work on my machine -- chris
//		tmpString.sprintf(" %s %s -o %s %s",keyringsToUse().data(), recipList,outFile,inFile);
//		pgpAppString.append(tmpString);
		retVal = runProcessFG(pgpAppString.data());
		break;
	default:
				showBadProfileDialog();
		break;
	}
	return retVal;
}

QStrList *myProfile::getEncryptOptions()
{
	QStrList *a = new QStrList();
	// DO NOT TRANSLATE THESE STRINGS - THEY ARE SENT AS-IS TO GPG/PGP!
	switch (pgpVersion) {
		case 1:
			a->append("--verbose");
			a->append("--quiet");
			a->append("--no-tty");
			a->append("--force-v3-sigs");
			a->append("--force-mdc");
			a->append("--dry-run");
			a->append("--batch");
			a->append("--yes");
			a->append("--no");
			a->append("--rfc1991");
			a->append("--openpgp");
			a->append("--throw-keyid");
	}
	return a;
}

