
#ifndef PGPPROFILE_H
#define PGPPROFILE_H

#include "includes.h"
#include "defines.h"
#include "myClipboard.h"
#include "myKeyring.h"

// -----------------------------------------------------------------

// Note: myProfile should NEVER have to open a GUI entity unless absolutely necessary!
// Pretend it's a library and you make calls to it. Thanks, Chris. 

class myProfile {
	public:
		myProfile(const char *newProfileName); // for existing profiles
		myProfile(int newPGPVersion, const char *pathFolder); // for new profiles
		~myProfile();

		QString keyringsToUse(const bool bIncludeSecret=TRUE, const bool bIncludePublic=TRUE);
	//	char *keyringsToUse(const bool bIncludeSecret=TRUE, const bool bIncludePublic=TRUE);
		void loadKeyring(const bool justRefreshTrust=FALSE, const char *keyID=NULL);
		int editKey(const char *keyID);
		int signKey(const char *keyIDToSign, const char *signingKeyID);
		int revokeSignature(const char *sigID);
		int removeRevokeKey(const char *keyID);
		int disEnableKey(const char *keyID);
		int removeKey(const char *keyID);
		int revokeKey(const char *keyID);
		int addName(const char *keyID);
		int genKeyPair();
		int importKey_Server(const char *keyID, bool forceFullUpdate=TRUE); // requires a KeyID
		int importKey_Server(QStrList *keyList, bool forceFullUpdate=TRUE); // supports multiple keyIDs
		int importKey_File(const char *fileName);    // requires a file path
		int importKey_Clipboard();
		int exportKey_Server(const char *keyID);
		int exportKey_File(const char *keyID, const char *fileName);
		int exportKey_Clipboard(const char *keyID);
		pkiKey *getKey(const char *keyID);
		pkiSubKey *getSubKey(const char *subKeyID, QList<pkiSubKey> *subkeys);
		pkiSig *getSig(const char *keyID, QList<pkiSig> *sigs);
		QStrList *getEncryptOptions();
		int decryptFile(const char *inFile, const char *outFile, bool detachSig, bool justVerify, QStrList *optionList=NULL);
		int encryptFile(const char *inFile, const char *outFile, const char *signingID, const char *recipList,
			bool detachSig, bool signFile, bool encryptFile, bool asciiArmor, bool uniText, bool clearSign, QStrList *optionList=NULL);

	// Public variables:
		int pgpVersion;
			// PGP 7.0 = 7
			// PGP 6.5 = 6
			// PGP 5   = 5
			// PGP 2.6 = 2
			// GnuPG   = 1
		QString profileName;
		QString keyServer; 
		bool keyServerCompatible;
		QString recipFormat; // printf(recipFormat,recipientID);
		QStrList pgpExecs;
			// PGP 6.5: pgp
			// PGP 5: pgpk, pgpe, pgps, pgpv
			// PGP 2: pgp
			// GPG 1: gpg
		QStrList ciphers; // Cipher: 3DES, CAST5, BLOWFISH ** NOT USED **
		QStrList keyTypes; // Pubkey: ELG-E, DSA, ELG ** NOT USED **
		QStrList hashes; // Hash: MD5, SHA1, RIPEMD160 ** NOT USED **
		QStrList secretKeyringFiles;
		QStrList publicKeyringFiles;
		QList<pkiKey> keyring;
        
	private:
		void loadProfile(const char *profileName);

		void verifyDecryptFile(int mode, char *file1, char *file2);

		void loadGPGKeyring(const char *appString, int mode, QProgressDialog *winProgress);
		
 		void loadPGP6Keyring(const char *appString, int mode, QProgressDialog *winProgress);
		void loadPGP5Keyring(const char *appString, QProgressDialog *winProgress);
 		void loadPGP65Validity(const char *appString, QProgressDialog *winProgress);

		void loadPGP2Keyring(const char *appString, int mode, QProgressDialog *winProgress);
		void loadPGP2Validity(const char *appString, QProgressDialog *winProgress);
};

#endif
